@@ -10,16 +10,9 @@ o PSSM
 """
 
 # standard library
-import string
 import math
 import sys
 
-#TODO - Remove this work around once we drop python 2.3 support
-try:
-   set = set
-except NameError:
-   from sets import Set as set
-
 # biopython modules
 from Bio import Alphabet
 from Bio.Alphabet import IUPAC
@@ -89,8 +82,7 @@ class SummaryInfo:
                         if record.seq[n] not in atom_dict.keys():
                             atom_dict[record.seq[n]] = 1
                         else:
-                            atom_dict[record.seq[n]] = \
-                              atom_dict[record.seq[n]] + 1
+                            atom_dict[record.seq[n]] += 1
 
                         num_atoms = num_atoms + 1
 
@@ -105,12 +97,12 @@ class SummaryInfo:
                     max_atoms.append(atom)
 
             if require_multiple and num_atoms == 1:
-                consensus = consensus + ambiguous
+                consensus += ambiguous
             elif (len(max_atoms) == 1) and ((float(max_size)/float(num_atoms))
                                          >= threshold):
-                consensus = consensus + max_atoms[0]
+                consensus += max_atoms[0]
             else:
-                consensus = consensus + ambiguous
+                consensus += ambiguous
 
         # we need to guess a consensus alphabet if one isn't specified
         if consensus_alpha is None:
@@ -122,9 +114,9 @@ class SummaryInfo:
                        consensus_alpha = None, require_multiple = 0):
         """Same as dumb_consensus(), but allows gap on the output.
 
-	Things to do: Let the user define that with only one gap, the result
-	character in consensus is gap. Let the user select gap character, now
-	it takes the same is input.
+        Things to do: Let the user define that with only one gap, the result
+        character in consensus is gap. Let the user select gap character, now
+        it takes the same is input.
         """
         # Iddo Friedberg, 1-JUL-2004: changed ambiguous default to "X"
         consensus = ''
@@ -145,10 +137,9 @@ class SummaryInfo:
                     if record.seq[n] not in atom_dict.keys():
                         atom_dict[record.seq[n]] = 1
                     else:
-                        atom_dict[record.seq[n]] = \
-                          atom_dict[record.seq[n]] + 1
+                        atom_dict[record.seq[n]] += 1
 
-                    num_atoms = num_atoms + 1
+                    num_atoms += 1
 
             max_atoms = []
             max_size = 0
@@ -161,12 +152,12 @@ class SummaryInfo:
                     max_atoms.append(atom)
 
             if require_multiple and num_atoms == 1:
-                consensus = consensus + ambiguous
+                consensus += ambiguous
             elif (len(max_atoms) == 1) and ((float(max_size)/float(num_atoms))
                                          >= threshold):
-                consensus = consensus + max_atoms[0]
+                consensus += max_atoms[0]
             else:
-                consensus = consensus + ambiguous
+                consensus += ambiguous
 
         # we need to guess a consensus alphabet if one isn't specified
         if consensus_alpha is None:
@@ -186,34 +177,34 @@ class SummaryInfo:
         a = Alphabet._get_base_alphabet(self.alignment._alphabet)
 
         #Now check its compatible with all the rest of the sequences
-        for record in self.alignment :
+        for record in self.alignment:
             #Get the (un-gapped version of) the sequence's alphabet
             alt =  Alphabet._get_base_alphabet(record.seq.alphabet)
-            if not isinstance(alt, a.__class__) :
+            if not isinstance(alt, a.__class__):
                 raise ValueError \
                 ("Alignment contains a sequence with an incompatible alphabet.")
 
         #Check the ambiguous character we are going to use in the consensus
         #is in the alphabet's list of valid letters (if defined).
         if hasattr(a, "letters") and a.letters is not None \
-        and ambiguous not in a.letters :
+        and ambiguous not in a.letters:
             #We'll need to pick a more generic alphabet...
-            if isinstance(a, IUPAC.IUPACUnambiguousDNA) :
-                if ambiguous in IUPAC.IUPACUnambiguousDNA().letters :
+            if isinstance(a, IUPAC.IUPACUnambiguousDNA):
+                if ambiguous in IUPAC.IUPACUnambiguousDNA().letters:
                     a = IUPAC.IUPACUnambiguousDNA()
-                else :
+                else:
                     a = Alphabet.generic_dna
-            elif isinstance(a, IUPAC.IUPACUnambiguousRNA) :
-                if ambiguous in IUPAC.IUPACUnambiguousRNA().letters :
+            elif isinstance(a, IUPAC.IUPACUnambiguousRNA):
+                if ambiguous in IUPAC.IUPACUnambiguousRNA().letters:
                     a = IUPAC.IUPACUnambiguousRNA()
-                else :
+                else:
                     a = Alphabet.generic_rna
-            elif isinstance(a, IUPAC.IUPACProtein) :
-                if ambiguous in IUPAC.ExtendedIUPACProtein().letters :
+            elif isinstance(a, IUPAC.IUPACProtein):
+                if ambiguous in IUPAC.ExtendedIUPACProtein().letters:
                     a = IUPAC.ExtendedIUPACProtein()
-                else :
+                else:
                     a = Alphabet.generic_protein
-            else :
+            else:
                 a = Alphabet.single_letter_alphabet
         return a
 
@@ -262,8 +253,8 @@ class SummaryInfo:
                 rep_dict = self._pair_replacement(
                     self.alignment._records[rec_num1].seq,
                     self.alignment._records[rec_num2].seq,
-                    self.alignment._records[rec_num1].annotations.get('weight',1),
-                    self.alignment._records[rec_num2].annotations.get('weight',1),
+                    self.alignment._records[rec_num1].annotations.get('weight',1.0),
+                    self.alignment._records[rec_num2].annotations.get('weight',1.0),
                     rep_dict, skip_items)
 
         return rep_dict
@@ -299,9 +290,7 @@ class SummaryInfo:
                 try:
                     # add info about the replacement to the dictionary,
                     # modified by the sequence weights
-                    start_dict[(residue1, residue2)] = \
-                                         start_dict[(residue1, residue2)] + \
-                                         weight1 * weight2
+                    start_dict[(residue1, residue2)] += weight1 * weight2
                                          
                 # if we get a key error, then we've got a problem with alphabets
                 except KeyError:
@@ -322,7 +311,7 @@ class SummaryInfo:
             #letters are not defined!  We must build a list of the
             #letters used...
             set_letters = set()
-            for record in self.alignment :
+            for record in self.alignment:
                 #Note the built in set does not have a union_update
                 #which was provided by the sets module's Set
                 set_letters = set_letters.union(record.seq)
@@ -352,9 +341,7 @@ class SummaryInfo:
         # and drop it out
         if isinstance(self.alignment._alphabet, Alphabet.Gapped):
             skip_items.append(self.alignment._alphabet.gap_char)
-            all_letters = string.replace(all_letters,
-                                         self.alignment._alphabet.gap_char,
-                                         '')
+            all_letters = all_letters.replace(self.alignment._alphabet.gap_char,'')
 
         # now create the dictionary
         for first_letter in all_letters:
@@ -389,7 +376,7 @@ class SummaryInfo:
         all_letters = self._get_all_letters()
         assert all_letters
 
-        if not isinstance(chars_to_ignore, list) :
+        if not isinstance(chars_to_ignore, list):
             raise TypeError("chars_to_ignore should be a list.")
 
         # if we have a gap char, add it to stuff to ignore
@@ -397,7 +384,7 @@ class SummaryInfo:
             chars_to_ignore.append(self.alignment._alphabet.gap_char)
         
         for char in chars_to_ignore:
-            all_letters = string.replace(all_letters, char, '')
+            all_letters = all_letters.replace(char, '')
 
         if axis_seq:
             left_seq = axis_seq
@@ -418,7 +405,7 @@ class SummaryInfo:
                     this_residue = None
                     
                 if this_residue and this_residue not in chars_to_ignore:
-                    weight = record.annotations.get('weight', 1)
+                    weight = record.annotations.get('weight', 1.0)
                     try:
                         score_dict[this_residue] += weight
                     # if we get a KeyError then we have an alphabet problem
@@ -483,23 +470,23 @@ class SummaryInfo:
         if not e_freq_table:
             #TODO - What about ambiguous alphabets?
             base_alpha = Alphabet._get_base_alphabet(self.alignment._alphabet)
-            if isinstance(base_alpha, Alphabet.ProteinAlphabet) :
+            if isinstance(base_alpha, Alphabet.ProteinAlphabet):
                 random_expected = Protein20Random
-            elif isinstance(base_alpha, Alphabet.NucleotideAlphabet) :
+            elif isinstance(base_alpha, Alphabet.NucleotideAlphabet):
                 random_expected = Nucleotide4Random
-            else :
+            else:
                 errstr = "Error in alphabet: not Nucleotide or Protein, "
                 errstr += "supply expected frequencies"
                 raise ValueError(errstr)
             del base_alpha
-        elif not isinstance(e_freq_table, FreqTable.FreqTable) :
+        elif not isinstance(e_freq_table, FreqTable.FreqTable):
             raise ValueError("e_freq_table should be a FreqTable object")
             
 
         # determine all of the letters we have to deal with
         all_letters = self._get_all_letters()
         for char in chars_to_ignore:
-            all_letters = string.replace(all_letters, char, '')
+            all_letters = all_letters.replace(char, '')
 
         info_content = {}
         for residue_num in range(start, end):
@@ -516,7 +503,7 @@ class SummaryInfo:
         # sum up the score
         total_info = 0
         for column_info in info_content.values():
-            total_info = total_info + column_info
+            total_info += column_info
         # fill in the ic_vector member: holds IC for each column
         for i in info_content.keys():
             self.ic_vector[i] = info_content[i]
@@ -545,7 +532,7 @@ class SummaryInfo:
         for record in all_records:
             try:
                 if record.seq[residue_num] not in to_ignore:
-                    weight = record.annotations.get('weight',1)
+                    weight = record.annotations.get('weight',1.0)
                     freq_info[record.seq[residue_num]] += weight
                     total_count += weight
             # getting a key error means we've got a problem with the alphabet 
@@ -554,12 +541,12 @@ class SummaryInfo:
                                  % (record.seq[residue_num],
                                     self.alignment._alphabet))
 
-        if total_count == 0 :
+        if total_count == 0:
             # This column must be entirely ignored characters
             for letter in freq_info.keys():
                 assert freq_info[letter] == 0
                 #TODO - Map this to NA or NaN?
-        else :
+        else:
             # now convert the counts into frequencies
             for letter in freq_info.keys():
                 freq_info[letter] = freq_info[letter] / total_count
@@ -577,15 +564,15 @@ class SummaryInfo:
         o log_base - The base of the logathrim to use in calculating the
         info content.
         """
-        try :
+        try:
             gap_char = self.alignment._alphabet.gap_char
-        except AttributeError :
+        except AttributeError:
             #The alphabet doesn't declare a gap - there could be none
             #in the sequence... or just a vague alphabet.
             gap_char = "-" #Safe?
             
         if e_freq_table:
-            if not isinstance(e_freq_table, FreqTable.FreqTable) :
+            if not isinstance(e_freq_table, FreqTable.FreqTable):
                 raise ValueError("e_freq_table should be a FreqTable object")
             # check the expected freq information to make sure it is good
             for key in obs_freq.keys():
@@ -612,7 +599,7 @@ class SummaryInfo:
             if inner_log > 0:
                 letter_info = (obs_freq[letter] * 
                                math.log(inner_log) / math.log(log_base))
-                total_info = total_info + letter_info
+                total_info += letter_info
         return total_info 
 
     def get_column(self,col):
@@ -668,16 +655,16 @@ class PSSM:
         
         # first print out the top header
         for res in all_residues:
-            out = out + "   %s" % res
-        out = out + "\n"
+            out += "   %s" % res
+        out += "\n"
 
         # for each item, write out the substitutions
         for item in self.pssm:
-            out = out + "%s " % item[0]
+            out += "%s " % item[0]
             for res in all_residues:
-                out = out + " %.1f" % item[1][res]
+                out += " %.1f" % item[1][res]
 
-            out = out + "\n"
+            out += "\n"
         return out
 
     def get_residue(self, pos):
@@ -699,7 +686,7 @@ def print_info_content(summary_info,fout=None,rep_record=0):
         fout.write("%d %s %.3f\n" % (pos, rep_sequence[pos],
                    summary_info.ic_vector[pos]))
 
-if __name__ == "__main__" :
+if __name__ == "__main__":
     print "Quick test"
     from Bio import AlignIO
     from Bio.Align.Generic import Alignment
@@ -711,7 +698,7 @@ if __name__ == "__main__" :
                                    IUPAC.unambiguous_dna)
 
     alignment = AlignIO.read(open(filename), format)
-    for record in alignment :
+    for record in alignment:
         print record.seq.tostring()
     print "="*alignment.get_alignment_length()
     
