@@ -1,4 +1,4 @@
-# Copyright 2008 by Peter Cock.  All rights reserved.
+# Copyright 2008-2009 by Peter Cock.  All rights reserved.
 #
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
@@ -16,7 +16,7 @@ example from the alignret, water and needle tools.
 from Bio.Align.Generic import Alignment
 from Interfaces import AlignmentIterator, SequentialAlignmentWriter
 
-class EmbossWriter(SequentialAlignmentWriter) :
+class EmbossWriter(SequentialAlignmentWriter):
     """Emboss alignment writer (WORK IN PROGRESS).
 
     Writes a simplfied version of the EMBOSS pairs/simple file format.
@@ -24,22 +24,22 @@ class EmbossWriter(SequentialAlignmentWriter) :
     available and is ommitted.
     """
 
-    def write_header(self) :
+    def write_header(self):
         handle = self.handle
         handle.write("########################################\n")
         handle.write("# Program: Biopython\n")
-        try :
+        try:
             handle.write("# Report_file: %s\n" % handle.name)
-        except AttributeError :
+        except AttributeError:
             pass
         handle.write("########################################\n")
 
-    def write_footer(self) :
+    def write_footer(self):
         handle = self.handle
         handle.write("#---------------------------------------\n")
         handle.write("#---------------------------------------\n")
         
-    def write_alignment(self, alignment) :
+    def write_alignment(self, alignment):
         """Use this to write (another) single alignment to an open file."""
 
         handle = self.handle
@@ -48,7 +48,7 @@ class EmbossWriter(SequentialAlignmentWriter) :
         handle.write("#=======================================\n")
         handle.write("#\n")
         handle.write("# Aligned_sequences: %i\n" % len(records))
-        for i, record in enumerate(records) :
+        for i, record in enumerate(records):
             handle.write("# %i: %s\n" % (i+1, record.id))
         handle.write("#\n")
         handle.write("# Length: %i\n" % alignment.get_alignment_length())
@@ -58,18 +58,18 @@ class EmbossWriter(SequentialAlignmentWriter) :
         #...
         assert False
 
-class EmbossIterator(AlignmentIterator) :
+class EmbossIterator(AlignmentIterator):
     """Emboss alignment iterator.
 
     For reading the (pairwise) alignments from EMBOSS tools in what they
     call the "pairs" and "simple" formats.
     """
     
-    def next(self) :
+    def next(self):
 
         handle = self.handle
 
-        try :
+        try:
             #Header we saved from when we were parsing
             #the previous alignment.
             line = self._header
@@ -79,9 +79,9 @@ class EmbossIterator(AlignmentIterator) :
         if not line:
             return None
 
-        while line.rstrip() != "#=======================================" :
+        while line.rstrip() != "#=======================================":
             line = handle.readline()
-            if not line :
+            if not line:
                 return None
 
         length_of_seqs = None
@@ -90,44 +90,45 @@ class EmbossIterator(AlignmentIterator) :
         seqs = []
 
 
-        while line[0] == "#" :
+        while line[0] == "#":
             #Read in the rest of this alignment header,
             #try and discover the number of records expected
             #and their length
             parts = line[1:].split(":",1)
             key = parts[0].lower().strip()
-            if key == "aligned_sequences" :
+            if key == "aligned_sequences":
                 number_of_seqs = int(parts[1].strip())
                 assert len(ids) == 0
                 # Should now expect the record identifiers...
-                for i in range(number_of_seqs) :
+                for i in range(number_of_seqs):
                     line = handle.readline()
                     parts = line[1:].strip().split(":",1)
                     assert i+1 == int(parts[0].strip())
                     ids.append(parts[1].strip())
                 assert len(ids) == number_of_seqs
-            if key == "length" :
+            if key == "length":
                 length_of_seqs = int(parts[1].strip())
 
             #And read in another line...
             line = handle.readline()
 
-        if number_of_seqs is None :
-            raise SyntaxError("Number of sequences missing!")
-        if length_of_seqs is None :
-            raise SyntaxError("Length of sequences missing!")
+        if number_of_seqs is None:
+            raise ValueError("Number of sequences missing!")
+        if length_of_seqs is None:
+            raise ValueError("Length of sequences missing!")
 
         if self.records_per_alignment is not None \
-        and self.records_per_alignment != number_of_seqs :
+        and self.records_per_alignment != number_of_seqs:
             raise ValueError("Found %i records in this alignment, told to expect %i" \
                              % (number_of_seqs, self.records_per_alignment))
 
         seqs = ["" for id in ids]
+        seq_starts = []
         index = 0
 
         #Parse the seqs
-        while line :
-            if len(line) > 21 :
+        while line:
+            if len(line) > 21:
                 id_start = line[:21].strip().split(None, 1)
                 seq_end = line[21:].strip().split(None, 1)
                 if len(id_start) == 2 and len(seq_end) == 2:
@@ -135,6 +136,19 @@ class EmbossIterator(AlignmentIterator) :
                     #(an aligned seq is broken up into multiple lines)
                     id, start = id_start
                     seq, end = seq_end
+                    if start==end:
+                        #Special case, either a single letter is present,
+                        #or no letters at all.
+                        if seq.replace("-","") == "":
+                            start = int(start)
+                            end = int(end)
+                        else:
+                            start = int(start) - 1
+                            end = int(end)
+                    else:
+                        assert seq.replace("-","") != ""
+                        start = int(start)-1 #python counting
+                        end = int(end)
 
                     #The identifier is truncated...
                     assert 0 <= index and index < number_of_seqs, \
@@ -142,44 +156,44 @@ class EmbossIterator(AlignmentIterator) :
                            % (index, number_of_seqs)
                     assert id==ids[index] or id == ids[index][:len(id)]
 
+                    if len(seq_starts) == index:
+                        #Record the start
+                        seq_starts.append(start)
+
                     #Check the start...
-                    if int(start) == 0:
-                        #Special case when one sequence starts long before the other
-                        assert len(seqs[index].replace("-",""))==0
-                        assert len(seq.replace("-","")) == 0, line
-                    elif int(start) == len(seqs[index].replace("-","")) :
-                        #Special case when one sequence ends long before the other
-                        assert len(seq.replace("-","")) == 0, line
-                    else :
-                        assert int(start) - 1 == len(seqs[index].replace("-","")), \
-                        "Found %i chars so far for sequence %i (%s), file says start %i:\n%s" \
-                            % (len(seqs[index].replace("-","")), index, id,
-                               int(start), seqs[index])
+                    if start == end:
+                        assert seq.replace("-","") == "", line
+                    else:
+                        assert start - seq_starts[index] == len(seqs[index].replace("-","")), \
+                        "Found %i chars so far for sequence %i (%s, %s), line says start %i:\n%s" \
+                            % (len(seqs[index].replace("-","")), index, id, repr(seqs[index]),
+                               start, line)
                     
                     seqs[index] += seq
 
                     #Check the end ...
-                    assert int(end) == len(seqs[index].replace("-","")), \
-                        "Found %i chars so far for %s, file says end %i:\n%s" \
-                            % (len(seqs[index]), id, int(end), seqs[index])
+                    assert end == seq_starts[index] + len(seqs[index].replace("-","")), \
+                        "Found %i chars so far for sequence %i (%s, %s, start=%i), file says end %i:\n%s" \
+                            % (len(seqs[index].replace("-","")), index, id, repr(seqs[index]),
+                               seq_starts[index], end, line)
 
                     index += 1
-                    if index >= number_of_seqs :
+                    if index >= number_of_seqs:
                         index = 0
-                else :
+                else:
                     #just a start value, this is just alignment annotation (?)
                     #print "Skipping: " + line.rstrip()
                     pass
-            elif line.strip() == "" :
+            elif line.strip() == "":
                 #Just a spacer?
                 pass
-            else :
+            else:
                 print line
                 assert False
 
             line = handle.readline()
             if line.rstrip() == "#---------------------------------------" \
-            or line.rstrip() == "#=======================================" :
+            or line.rstrip() == "#=======================================":
                 #End of alignment
                 self._header = line
                 break
@@ -187,18 +201,24 @@ class EmbossIterator(AlignmentIterator) :
         assert index == 0
 
         if self.records_per_alignment is not None \
-        and self.records_per_alignment != len(ids) :
+        and self.records_per_alignment != len(ids):
             raise ValueError("Found %i records in this alignment, told to expect %i" \
                              % (len(ids), self.records_per_alignment))
 
         alignment = Alignment(self.alphabet)
-        for id, seq in zip(ids, seqs) :
-            if len(seq) != length_of_seqs :
-                raise SyntaxError("Error parsing alignment - sequences of different length?")
+        for id, seq in zip(ids, seqs):
+            if len(seq) != length_of_seqs:
+                #EMBOSS 2.9.0 is known to use spaces instead of minus signs
+                #for leading gaps, and thus fails to parse.  This old version
+                #is still used as of Dec 2008 behind the EBI SOAP webservice:
+                #http://www.ebi.ac.uk/Tools/webservices/wsdl/WSEmboss.wsdl
+                raise ValueError("Error parsing alignment - sequences of "
+                                 "different length? You could be using an "
+                                 "old version of EMBOSS.")
             alignment.add_sequence(id, seq)
         return alignment
     
-if __name__ == "__main__" :
+if __name__ == "__main__":
     print "Running a quick self-test"
 
     #http://emboss.sourceforge.net/docs/themes/alnformats/align.simple
@@ -580,12 +600,6 @@ asis             311 -----------------    311
     assert [r.id for r in alignments[1].get_all_seqs()] \
            == ["IXI_234", "IXI_235", "IXI_236", "IXI_237"]
 
-
-    #for a in EmbossIterator(StringIO(pair_example2)) :
-    #    print "Next:"
-    #    for r in a.get_all_seqs() :
-    #        print r.seq.tostring()[:20] + "...", r.id
-    
     alignments = list(EmbossIterator(StringIO(pair_example2)))
     assert len(alignments) == 5
     assert len(alignments[0].get_all_seqs()) == 2
@@ -595,7 +609,6 @@ asis             311 -----------------    311
            == ["ref_rec", "gi|94970041|receiver"]
 
 
-
     alignments = list(EmbossIterator(StringIO(pair_example3)))
     assert len(alignments) == 1
     assert len(alignments[0].get_all_seqs()) == 2
