@@ -1,4 +1,4 @@
-# Copyright 2006-2008 by Peter Cock.  All rights reserved.
+# Copyright 2006-2009 by Peter Cock.  All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
@@ -7,11 +7,135 @@ Bio.AlignIO support for the "stockholm" format (used in the PFAM database).
 
 You are expected to use this module via the Bio.AlignIO functions (or the
 Bio.SeqIO functions if you want to work directly with the gapped sequences).
+
+For example, consider a Stockholm alignment file containing the following::
+
+    # STOCKHOLM 1.0
+    #=GC SS_cons       .................<<<<<<<<...<<<<<<<........>>>>>>>..
+    AP001509.1         UUAAUCGAGCUCAACACUCUUCGUAUAUCCUC-UCAAUAUGG-GAUGAGGGU
+    #=GR AP001509.1 SS -----------------<<<<<<<<---..<<-<<-------->>->>..--
+    AE007476.1         AAAAUUGAAUAUCGUUUUACUUGUUUAU-GUCGUGAAU-UGG-CACGA-CGU
+    #=GR AE007476.1 SS -----------------<<<<<<<<-----<<.<<-------->>.>>----
+
+    #=GC SS_cons       ......<<<<<<<.......>>>>>>>..>>>>>>>>...............
+    AP001509.1         CUCUAC-AGGUA-CCGUAAA-UACCUAGCUACGAAAAGAAUGCAGUUAAUGU
+    #=GR AP001509.1 SS -------<<<<<--------->>>>>--->>>>>>>>---------------
+    AE007476.1         UUCUACAAGGUG-CCGG-AA-CACCUAACAAUAAGUAAGUCAGCAGUGAGAU
+    #=GR AE007476.1 SS ------.<<<<<--------->>>>>.-->>>>>>>>---------------
+    //
+
+This is a single multiple sequence alignment, so you would probably load this
+using the Bio.AlignIO.read() function:
+
+    >>> from Bio import AlignIO
+    >>> handle = open("Stockholm/simple.sth", "rU")
+    >>> align = AlignIO.read(handle, "stockholm")
+    >>> handle.close()
+    >>> print align
+    SingleLetterAlphabet() alignment with 2 rows and 104 columns
+    UUAAUCGAGCUCAACACUCUUCGUAUAUCCUC-UCAAUAUGG-G...UGU AP001509.1
+    AAAAUUGAAUAUCGUUUUACUUGUUUAU-GUCGUGAAU-UGG-C...GAU AE007476.1
+    >>> for record in align:
+    ...     print record.id, len(record)
+    AP001509.1 104
+    AE007476.1 104
+
+This example file is clearly using RNA, so you might want the Alignment object
+(and the SeqRecord objects it holds) to reflect this, rather than simple using
+the default single letter alphabet as shown above.  You can do this with an
+optional argument to the Bio.AlignIO.read() function:
+
+    >>> from Bio import AlignIO
+    >>> from Bio.Alphabet import generic_rna
+    >>> handle = open("Stockholm/simple.sth", "rU")
+    >>> align = AlignIO.read(handle, "stockholm", alphabet=generic_rna)
+    >>> handle.close()
+    >>> print align
+    RNAAlphabet() alignment with 2 rows and 104 columns
+    UUAAUCGAGCUCAACACUCUUCGUAUAUCCUC-UCAAUAUGG-G...UGU AP001509.1
+    AAAAUUGAAUAUCGUUUUACUUGUUUAU-GUCGUGAAU-UGG-C...GAU AE007476.1
+
+In addition to the sequences themselves, this example alignment also includes
+some GR lines for the secondary structure of the sequences.  These are strings,
+with one character for each letter in the associated sequence:
+
+    >>> for record in align:
+    ...     print record.id
+    ...     print record.seq
+    ...     print record.letter_annotations['secondary_structure']
+    AP001509.1
+    UUAAUCGAGCUCAACACUCUUCGUAUAUCCUC-UCAAUAUGG-GAUGAGGGUCUCUAC-AGGUA-CCGUAAA-UACCUAGCUACGAAAAGAAUGCAGUUAAUGU
+    -----------------<<<<<<<<---..<<-<<-------->>->>..---------<<<<<--------->>>>>--->>>>>>>>---------------
+    AE007476.1
+    AAAAUUGAAUAUCGUUUUACUUGUUUAU-GUCGUGAAU-UGG-CACGA-CGUUUCUACAAGGUG-CCGG-AA-CACCUAACAAUAAGUAAGUCAGCAGUGAGAU
+    -----------------<<<<<<<<-----<<.<<-------->>.>>----------.<<<<<--------->>>>>.-->>>>>>>>---------------
+
+Any general annotation for each row is recorded in the SeqRecord's annotations
+dictionary.  You can output this alignment in many different file formats using
+Bio.AlignIO.write(), or the Alignment object's format method:
+
+    >>> print align.format("fasta")
+    >AP001509.1
+    UUAAUCGAGCUCAACACUCUUCGUAUAUCCUC-UCAAUAUGG-GAUGAGGGUCUCUAC-A
+    GGUA-CCGUAAA-UACCUAGCUACGAAAAGAAUGCAGUUAAUGU
+    >AE007476.1
+    AAAAUUGAAUAUCGUUUUACUUGUUUAU-GUCGUGAAU-UGG-CACGA-CGUUUCUACAA
+    GGUG-CCGG-AA-CACCUAACAAUAAGUAAGUCAGCAGUGAGAU
+    <BLANKLINE>
+
+Most output formats won't be able to hold the annotation possible in a Stockholm file:
+
+    >>> print align.format("stockholm")
+    # STOCKHOLM 1.0
+    #=GF SQ 2
+    AP001509.1 UUAAUCGAGCUCAACACUCUUCGUAUAUCCUC-UCAAUAUGG-GAUGAGGGUCUCUAC-AGGUA-CCGUAAA-UACCUAGCUACGAAAAGAAUGCAGUUAAUGU
+    #=GS AP001509.1 AC AP001509.1
+    #=GS AP001509.1 DE AP001509.1
+    #=GR AP001509.1 SS -----------------<<<<<<<<---..<<-<<-------->>->>..---------<<<<<--------->>>>>--->>>>>>>>---------------
+    AE007476.1 AAAAUUGAAUAUCGUUUUACUUGUUUAU-GUCGUGAAU-UGG-CACGA-CGUUUCUACAAGGUG-CCGG-AA-CACCUAACAAUAAGUAAGUCAGCAGUGAGAU
+    #=GS AE007476.1 AC AE007476.1
+    #=GS AE007476.1 DE AE007476.1
+    #=GR AE007476.1 SS -----------------<<<<<<<<-----<<.<<-------->>.>>----------.<<<<<--------->>>>>.-->>>>>>>>---------------
+    //
+    <BLANKLINE>
+
+Note that when writing Stockholm files, Biopython does not break long sequences up and
+interleave them (as in the input file shown above).  The standard allows this simpler
+layout, and it is more likely to be understood by other tools. 
+
+Finally, as an aside, it can sometimes be useful to use Bio.SeqIO.parse() to iterate over
+the two rows as SeqRecord objects - rather than working with Alignnment objects.
+Again, if you want to you can specify this is RNA:
+
+    >>> from Bio import SeqIO
+    >>> from Bio.Alphabet import generic_rna
+    >>> handle = open("Stockholm/simple.sth", "rU")
+    >>> for record in SeqIO.parse(handle, "stockholm", alphabet=generic_rna):
+    ...     print record.id
+    ...     print record.seq
+    ...     print record.letter_annotations['secondary_structure']
+    AP001509.1
+    UUAAUCGAGCUCAACACUCUUCGUAUAUCCUC-UCAAUAUGG-GAUGAGGGUCUCUAC-AGGUA-CCGUAAA-UACCUAGCUACGAAAAGAAUGCAGUUAAUGU
+    -----------------<<<<<<<<---..<<-<<-------->>->>..---------<<<<<--------->>>>>--->>>>>>>>---------------
+    AE007476.1
+    AAAAUUGAAUAUCGUUUUACUUGUUUAU-GUCGUGAAU-UGG-CACGA-CGUUUCUACAAGGUG-CCGG-AA-CACCUAACAAUAAGUAAGUCAGCAGUGAGAU
+    -----------------<<<<<<<<-----<<.<<-------->>.>>----------.<<<<<--------->>>>>.-->>>>>>>>---------------
+    >>> handle.close()
+
+Remember that if you slice a SeqRecord, the per-letter-annotions like the
+secondary structure string here, are also sliced:
+
+    >>> sub_record = record[10:20]
+    >>> print sub_record.seq
+    AUCGUUUUAC
+    >>> print sub_record.letter_annotations['secondary_structure']
+    -------<<<
 """
+__docformat__ = "epytext en" #not just plaintext
 from Bio.Align.Generic import Alignment
 from Interfaces import AlignmentIterator, SequentialAlignmentWriter
 
-class StockholmWriter(SequentialAlignmentWriter) :
+class StockholmWriter(SequentialAlignmentWriter):
     """Stockholm/PFAM alignment writer."""
 
     #These dictionaries should be kept in sync with those
@@ -28,7 +152,7 @@ class StockholmWriter(SequentialAlignmentWriter) :
                        "organism_classification" : "OC",
                        "look" : "LO"}
 
-    def write_alignment(self, alignment) :
+    def write_alignment(self, alignment):
         """Use this to write (another) single alignment to an open file.
         
         Note that sequences and their annotation are recorded
@@ -44,42 +168,42 @@ class StockholmWriter(SequentialAlignmentWriter) :
         #NOTE - For now, the alignment object does not hold any per column
         #or per alignment annotation - only per sequence.
         
-        if count == 0 :
+        if count == 0:
             raise ValueError("Must have at least one sequence")
-        if self._length_of_sequences == 0 :
+        if self._length_of_sequences == 0:
             raise ValueError("Non-empty sequences are required")
 
         self.handle.write("# STOCKHOLM 1.0\n")
         self.handle.write("#=GF SQ %i\n" % count)
-        for record in records :
+        for record in records:
             self._write_record(record)
         self.handle.write("//\n")
 
     def _write_record(self, record):
         """Write a single SeqRecord to the file"""
-        if self._length_of_sequences != len(record.seq) :
+        if self._length_of_sequences != len(record.seq):
             raise ValueError("Sequences must all be the same length")
 
         #For the case for stockholm to stockholm, try and use record.name
         seq_name = record.id
-        if record.name is not None :
-            if "accession" in record.annotations :
-                if record.id == record.annotations["accession"] :
+        if record.name is not None:
+            if "accession" in record.annotations:
+                if record.id == record.annotations["accession"]:
                     seq_name = record.name
 
         #In the Stockholm file format, spaces are not allowed in the id
         seq_name = seq_name.replace(" ","_")
 
         if "start" in record.annotations \
-        and  "end" in record.annotations :
+        and  "end" in record.annotations:
             suffix = "/%s-%s" % (str(record.annotations["start"]),
                                  str(record.annotations["end"]))
-            if seq_name[-len(suffix):] != suffix :
+            if seq_name[-len(suffix):] != suffix:
                 seq_name = "%s/%s-%s" % (seq_name,
                                         str(record.annotations["start"]),
                                         str(record.annotations["end"]))
 
-        if seq_name in self._ids_written :
+        if seq_name in self._ids_written:
             raise ValueError("Duplicate record identifier: %s" % seq_name)
         self._ids_written.append(seq_name)
         self.handle.write("%s %s\n" % (seq_name, record.seq.tostring()))
@@ -96,45 +220,52 @@ class StockholmWriter(SequentialAlignmentWriter) :
         #us to write the file using a single pass through the records.
 
         #AC = Accession
-        if "accession" in record.annotations :
+        if "accession" in record.annotations:
             self.handle.write("#=GS %s AC %s\n" \
                 % (seq_name, self.clean(record.annotations["accession"])))
-        elif record.id :
+        elif record.id:
             self.handle.write("#=GS %s AC %s\n" \
                 % (seq_name, self.clean(record.id)))
         
         #DE = description
-        if record.description :
+        if record.description:
             self.handle.write("#=GS %s DE %s\n" \
                 % (seq_name, self.clean(record.description)))
 
         #DE = database links
-        for xref in record.dbxrefs :
+        for xref in record.dbxrefs:
             self.handle.write("#=GS %s DR %s\n" \
                 % (seq_name, self.clean(xref)))
 
-        #GS/GR = other per sequence annotation
-        for key in record.annotations :
-            if key in self.pfam_gs_mapping :
-                self.handle.write("#=GS %s %s %s\n" \
-                                  % (seq_name,
-                                     self.clean(self.pfam_gs_mapping[key]),
-                                     self.clean(str(record.annotations[key]))))
-            elif key in self.pfam_gr_mapping :
-                if len(str(record.annotations[key]))==len(record.seq) :
+        #GS = other per sequence annotation
+        for key, value in record.annotations.iteritems():
+            if key in self.pfam_gs_mapping:
+                data = self.clean(str(value))
+                if data:
+                    self.handle.write("#=GS %s %s %s\n" \
+                                      % (seq_name,
+                                         self.clean(self.pfam_gs_mapping[key]),
+                                         data))
+            else:
+                #It doesn't follow the PFAM standards, but should we record
+                #this data anyway?
+                pass
+
+        #GR = per row per column sequence annotation
+        for key, value in record.letter_annotations.iteritems():
+            if key in self.pfam_gr_mapping and len(str(value))==len(record.seq):
+                data = self.clean(str(value))
+                if data:
                     self.handle.write("#=GR %s %s %s\n" \
                                       % (seq_name,
                                          self.clean(self.pfam_gr_mapping[key]),
-                                         self.clean((record.annotations[key]))))
-                else :
-                    #Should we print a warning?
-                    pass
-            else :
+                                         data))
+            else:
                 #It doesn't follow the PFAM standards, but should we record
                 #this data anyway?
                 pass
         
-class StockholmIterator(AlignmentIterator) :
+class StockholmIterator(AlignmentIterator):
     """Loads a Stockholm file from PFAM into Alignment objects.
 
     The file may contain multiple concatenated alignments, which are loaded
@@ -178,11 +309,11 @@ class StockholmIterator(AlignmentIterator) :
                        "OC" : "organism_classification",
                        "LO" : "look"}
 
-    def next(self) :
-        try :
+    def next(self):
+        try:
             line = self._header
             del self._header
-        except AttributeError :
+        except AttributeError:
             line = self.handle.readline()
         if not line:
             #Empty file - just give up.
@@ -210,38 +341,38 @@ class StockholmIterator(AlignmentIterator) :
             if line == '# STOCKHOLM 1.0':
                 self._header = line
                 break
-            elif line == "//" :
+            elif line == "//":
                 #The "//" line indicates the end of the alignment.
                 #There may still be more meta-data
                 passed_end_alignment = True
-            elif line == "" :
+            elif line == "":
                 #blank line, ignore
                 pass
-            elif line[0] != "#" :
+            elif line[0] != "#":
                 #Sequence
                 #Format: "<seqname> <sequence>"
                 assert not passed_end_alignment
                 parts = [x.strip() for x in line.split(" ",1)]
-                if len(parts) != 2 :
+                if len(parts) != 2:
                     #This might be someone attempting to store a zero length sequence?
                     raise ValueError("Could not split line into identifier " \
                                       + "and sequence:\n" + line)
                 id, seq = parts
-                if id not in ids :
+                if id not in ids:
                     ids.append(id)
                 seqs.setdefault(id, '')
                 seqs[id] += seq.replace(".","-")
-            elif len(line) >= 5 :
+            elif len(line) >= 5:
                 #Comment line or meta-data
-                if line[:5] == "#=GF " :
+                if line[:5] == "#=GF ":
                     #Generic per-File annotation, free text
                     #Format: #=GF <feature> <free text>
                     feature, text = line[5:].strip().split(None,1)
                     #Each feature key could be used more than once,
                     #so store the entries as a list of strings.
-                    if feature not in gf :
+                    if feature not in gf:
                         gf[feature] = [text]
-                    else :
+                    else:
                         gf[feature].append(text)
                 elif line[:5] == '#=GC ':
                     #Generic per-Column annotation, exactly 1 char per column
@@ -251,21 +382,21 @@ class StockholmIterator(AlignmentIterator) :
                     #Generic per-Sequence annotation, free text
                     #Format: "#=GS <seqname> <feature> <free text>"
                     id, feature, text = line[5:].strip().split(None,2)
-                    #if id not in ids :
+                    #if id not in ids:
                     #    ids.append(id)
-                    if id not in gs :
+                    if id not in gs:
                         gs[id] = {}
-                    if feature not in gs[id] :
+                    if feature not in gs[id]:
                         gs[id][feature] = [text]
-                    else :
+                    else:
                         gs[id][feature].append(text)
-                elif line[:5] == "#=GR " :
+                elif line[:5] == "#=GR ":
                     #Generic per-Sequence AND per-Column markup
                     #Format: "#=GR <seqname> <feature> <exactly 1 char per column>"
                     id, feature, text = line[5:].strip().split(None,2)
-                    #if id not in ids :
+                    #if id not in ids:
                     #    ids.append(id)
-                    if id not in gr :
+                    if id not in gr:
                         gr[id] = {}
                     if feature not in gr[id]:
                         gr[id][feature] = ""
@@ -285,10 +416,10 @@ class StockholmIterator(AlignmentIterator) :
         self.seq_annotation = gs
         self.seq_col_annotation = gr
 
-        if ids and seqs :
+        if ids and seqs:
 
             if self.records_per_alignment is not None \
-            and self.records_per_alignment != len(ids) :
+            and self.records_per_alignment != len(ids):
                 raise ValueError("Found %i records in this alignment, told to expect %i" \
                                  % (len(ids), self.records_per_alignment))
 
@@ -299,9 +430,9 @@ class StockholmIterator(AlignmentIterator) :
             alignment._annotations = gr
 
             alignment_length = len(seqs.values()[0])
-            for id in ids :
+            for id in ids:
                 seq = seqs[id]
-                if alignment_length != len(seq) :
+                if alignment_length != len(seq):
                     raise ValueError("Sequences have different lengths, or repeated identifier")
                 name, start, end = self._identifier_split(id)
                 alignment.add_sequence(id, seq, start=start, end=end)
@@ -320,21 +451,21 @@ class StockholmIterator(AlignmentIterator) :
 
                 self._populate_meta_data(id, record)
             return alignment
-        else :
+        else:
             return None
 
 
-    def _identifier_split(self, identifier) :
+    def _identifier_split(self, identifier):
         """Returns (name,start,end) string tuple from an identier."""
-        if identifier.find("/")!=-1 :
+        if identifier.find("/")!=-1:
             start_end = identifier.split("/",1)[1]
-            if start_end.count("-")==1 :
+            if start_end.count("-")==1:
                 start, end = map(int, start_end.split("-"))
                 name = identifier.split("/",1)[0]
                 return (name, start, end)
         return (identifier, None, None)
     
-    def _get_meta_data(self, identifier, meta_dict) :
+    def _get_meta_data(self, identifier, meta_dict):
         """Takes an itentifier and returns dict of all meta-data matching it.
 
         For example, given "Q9PN73_CAMJE/149-220" will return all matches to
@@ -342,38 +473,38 @@ class StockholmIterator(AlignmentIterator) :
         suffix.
 
         In the example below, the suffix is required to match the AC, but must
-        be removed to match the OS and OC meta-data.
+        be removed to match the OS and OC meta-data::
 
-        # STOCKHOLM 1.0
-        #=GS Q9PN73_CAMJE/149-220  AC Q9PN73
-        ...
-        Q9PN73_CAMJE/149-220               NKA...
-        ...
-        #=GS Q9PN73_CAMJE OS Campylobacter jejuni
-        #=GS Q9PN73_CAMJE OC Bacteria 
+            # STOCKHOLM 1.0
+            #=GS Q9PN73_CAMJE/149-220  AC Q9PN73
+            ...
+            Q9PN73_CAMJE/149-220               NKA...
+            ...
+            #=GS Q9PN73_CAMJE OS Campylobacter jejuni
+            #=GS Q9PN73_CAMJE OC Bacteria 
 
         This function will return an empty dictionary if no data is found."""
         name, start, end = self._identifier_split(identifier)
-        if name==identifier :
+        if name==identifier:
             identifier_keys = [identifier]
-        else :
+        else:
             identifier_keys = [identifier, name]
         answer = {}
-        for identifier_key in identifier_keys :
-            try :
-                for feature_key in meta_dict[identifier_key] :
+        for identifier_key in identifier_keys:
+            try:
+                for feature_key in meta_dict[identifier_key]:
                     answer[feature_key] = meta_dict[identifier_key][feature_key]
-            except KeyError :
+            except KeyError:
                 pass
         return answer
 
-    def _populate_meta_data(self, identifier, record) :
+    def _populate_meta_data(self, identifier, record):
         """Adds meta-date to a SecRecord's annotations dictionary.
 
         This function applies the PFAM conventions."""
 
         seq_data = self._get_meta_data(identifier, self.seq_annotation)
-        for feature in seq_data :
+        for feature in seq_data:
             #Note this dictionary contains lists!
             if feature=="AC" : #ACcession number
                 assert len(seq_data[feature])==1
@@ -383,162 +514,39 @@ class StockholmIterator(AlignmentIterator) :
             elif feature=="DR" : #Database Reference
                 #Should we try and parse the strings?
                 record.dbxrefs = seq_data[feature]
-            elif feature in self.pfam_gs_mapping :
+            elif feature in self.pfam_gs_mapping:
                 record.annotations[self.pfam_gs_mapping[feature]] = ", ".join(seq_data[feature])
-            else :
+            else:
                 #Ignore it?
                 record.annotations["GS:" + feature] = ", ".join(seq_data[feature])
 
+        #Now record the per-letter-annotations
         seq_col_data = self._get_meta_data(identifier, self.seq_col_annotation)
-        for feature in seq_col_data :
+        for feature in seq_col_data:
             #Note this dictionary contains strings!
-            if feature in self.pfam_gr_mapping :
-                record.annotations[self.pfam_gr_mapping[feature]] = seq_col_data[feature]
-            else :
+            if feature in self.pfam_gr_mapping:
+                record.letter_annotations[self.pfam_gr_mapping[feature]] = seq_col_data[feature]
+            else:
                 #Ignore it?
-                record.annotations["GR:" + feature] = seq_col_data[feature]
+                record.letter_annotations["GR:" + feature] = seq_col_data[feature]
     
-if __name__ == "__main__" :
-    print "Testing..."
-    from cStringIO import StringIO
+def _test():
+    """Run the Bio.SeqIO module's doctests.
 
-    # This example with its slightly odd (partial) annotation is from here:
-    # http://www.cgb.ki.se/cgb/groups/sonnhammer/Stockholm.html
-    # I don't know what the "GR_O31699/88-139_IN ..." line is meant to be.
-    sth_example = \
-"""# STOCKHOLM 1.0
-#=GF ID CBS
-#=GF AC PF00571
-#=GF DE CBS domain
-#=GF AU Bateman A
-#=GF CC CBS domains are small intracellular modules mostly found  
-#=GF CC in 2 or four copies within a protein. 
-#=GF SQ 67
-#=GS O31698/18-71 AC O31698
-#=GS O83071/192-246 AC O83071
-#=GS O83071/259-312 AC O83071
-#=GS O31698/88-139 AC O31698
-#=GS O31698/88-139 OS Bacillus subtilis
-O83071/192-246          MTCRAQLIAVPRASSLAE..AIACAQKM....RVSRVPVYERS
-#=GR O83071/192-246 SA  999887756453524252..55152525....36463774777
-O83071/259-312          MQHVSAPVFVFECTRLAY..VQHKLRAH....SRAVAIVLDEY
-#=GR O83071/259-312 SS  CCCCCHHHHHHHHHHHHH..EEEEEEEE....EEEEEEEEEEE
-O31698/18-71            MIEADKVAHVQVGNNLEH..ALLVLTKT....GYTAIPVLDPS
-#=GR O31698/18-71 SS    CCCHHHHHHHHHHHHHHH..EEEEEEEE....EEEEEEEEHHH
-O31698/88-139           EVMLTDIPRLHINDPIMK..GFGMVINN......GFVCVENDE
-#=GR O31698/88-139 SS   CCCCCCCHHHHHHHHHHH..HEEEEEEE....EEEEEEEEEEH
-#=GC SS_cons            CCCCCHHHHHHHHHHHHH..EEEEEEEE....EEEEEEEEEEH
-O31699/88-139           EVMLTDIPRLHINDPIMK..GFGMVINN......GFVCVENDE
-#=GR O31699/88-139 AS   ________________*__________________________
-#=GR_O31699/88-139_IN   ____________1______________2__________0____
-//
-"""
-
-    # Interlaced example from BioPerl documentation.  Also note the blank line.
-    # http://www.bioperl.org/wiki/Stockholm_multiple_alignment_format
-    sth_example2 = \
-"""# STOCKHOLM 1.0
-#=GC SS_cons       .................<<<<<<<<...<<<<<<<........>>>>>>>..
-AP001509.1         UUAAUCGAGCUCAACACUCUUCGUAUAUCCUC-UCAAUAUGG-GAUGAGGGU
-#=GR AP001509.1 SS -----------------<<<<<<<<---..<<-<<-------->>->>..--
-AE007476.1         AAAAUUGAAUAUCGUUUUACUUGUUUAU-GUCGUGAAU-UGG-CACGA-CGU
-#=GR AE007476.1 SS -----------------<<<<<<<<-----<<.<<-------->>.>>----
-
-#=GC SS_cons       ......<<<<<<<.......>>>>>>>..>>>>>>>>...............
-AP001509.1         CUCUAC-AGGUA-CCGUAAA-UACCUAGCUACGAAAAGAAUGCAGUUAAUGU
-#=GR AP001509.1 SS -------<<<<<--------->>>>>--->>>>>>>>---------------
-AE007476.1         UUCUACAAGGUG-CCGG-AA-CACCUAACAAUAAGUAAGUCAGCAGUGAGAU
-#=GR AE007476.1 SS ------.<<<<<--------->>>>>.-->>>>>>>>---------------
-//"""
-
-    print "--------"
-    print "StockholmIterator(stockholm alignment file)"
-    iterator = StockholmIterator(StringIO(sth_example))
-    count=0
-    for alignment in iterator :
-        for record in alignment.get_all_seqs() :
-            count=count+1
-    assert count == 5
-    
-    #Check the last record...
-    assert record.id == 'O31699/88-139'
-    assert record.name == 'O31699'
-    assert record.description == 'O31699/88-139'
-    assert len(record.annotations)==4+1 #weight
-    assert record.annotations["accession"]=='O31699'
-    assert record.annotations["start"]==88
-    assert record.annotations["end"]==139
-    assert record.annotations["active_site"]=='________________*__________________________'
-
-    iterator = StockholmIterator(StringIO(sth_example))
-    count=0
-    for alignment in iterator :
-        for record in alignment.get_all_seqs() :
-            count=count+1
-            break
-        break
-    assert count==1
-    #Check the last record...
-    assert record.id == 'O83071/192-246'
-    assert record.name == 'O83071'
-    assert record.description == 'O83071/192-246'
-    assert len(record.annotations)==4 + 1#weight
-    assert record.annotations["accession"]=='O83071'
-    assert record.annotations["start"]==192
-    assert record.annotations["end"]==246
-    assert record.annotations["surface_accessibility"]=="999887756453524252..55152525....36463774777"
-
-    assert [[r.id for r in a.get_all_seqs()] \
-            for a in StockholmIterator(StringIO(sth_example))] \
-        == [['O83071/192-246', 'O83071/259-312', 'O31698/18-71', 'O31698/88-139', 'O31699/88-139']]
-
-    assert [[r.name for r in a.get_all_seqs()] \
-            for a in StockholmIterator(StringIO(sth_example))] \
-        == [['O83071', 'O83071', 'O31698', 'O31698', 'O31699']]
-
-    assert [[r.description for r in a.get_all_seqs()] \
-            for a in StockholmIterator(StringIO(sth_example))] \
-        == [['O83071/192-246', 'O83071/259-312', 'O31698/18-71', 'O31698/88-139', 'O31699/88-139']]
-
-
-    print "--------"
-    print "StockholmIterator(interlaced stockholm alignment file)"
-    iterator = iter(StockholmIterator(StringIO(sth_example2)).next().get_all_seqs())
-    record = iterator.next()
-    assert record.id == "AP001509.1"
-    assert len(record.seq) == 104
-    assert "secondary_structure" in record.annotations
-    assert len(record.annotations["secondary_structure"]) == 104
-    record = iterator.next()
-    assert record.id == "AE007476.1"
-    assert len(record.seq) == 104
-    assert "secondary_structure" in record.annotations
-    assert len(record.annotations["secondary_structure"]) == 104
-    try :
-        record = iterator.next()
-    except StopIteration :
-        record = None
-    assert record is None
-    
-    print "--------"
-    print "StockholmIterator(concatenated stockholm alignment files)"
-    assert 2 == len(list(StockholmIterator(StringIO(sth_example + sth_example2))))
-    assert 2 == len(list(StockholmIterator(StringIO(sth_example + "\n" + sth_example2))))
-    assert 2 == len(list(StockholmIterator(StringIO(sth_example + "\n\n" + sth_example2))))
-    assert 2 == len(list(StockholmIterator(StringIO(sth_example2 + "\n\n" + sth_example))))
-    assert 2 == len(list(StockholmIterator(StringIO(sth_example2 + "\n" + sth_example))))
-
-    print "--------"
-    print "Checking write/read"
-    handle = StringIO()
-    list1 = list(StockholmIterator(StringIO(sth_example2 + "\n" + sth_example)))
-    StockholmWriter(handle).write_file(list1)
-    handle.seek(0)
-    list2 = list(StockholmIterator(handle))
-    assert len(list1) == len(list2)
-    for a1,a2 in zip(list1, list2) :
-        assert len(a1.get_all_seqs()) == len(a2.get_all_seqs())
-        for r1, r2 in zip(a1.get_all_seqs(), a2.get_all_seqs()) :
-            assert r1.id == r2.id
-            assert r1.seq.tostring() == r2.seq.tostring()
-    print "Done"
+    This will try and locate the unit tests directory, and run the doctests
+    from there in order that the relative paths used in the examples work.
+    """
+    import doctest
+    import os
+    if os.path.isdir(os.path.join("..","..","Tests")):
+        print "Runing doctests..."
+        cur_dir = os.path.abspath(os.curdir)
+        os.chdir(os.path.join("..","..","Tests"))
+        assert os.path.isfile("Stockholm/simple.sth")
+        doctest.testmod()
+        os.chdir(cur_dir)
+        del cur_dir
+        print "Done"
+        
+if __name__ == "__main__":
+    _test()
