@@ -1,4 +1,4 @@
-# Copyright 2008 by Peter Cock.  All rights reserved.
+# Copyright 2008-2009 by Peter Cock.  All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
@@ -11,6 +11,11 @@ format names (lower case strings).  From the user's perspective, you can read
 in a PHYLIP file containing one or more alignments using Bio.AlignIO, or you
 can read in the sequences within these alignmenta using Bio.SeqIO.
 
+Bio.AlignIO is also documented at U{http://biopython.org/wiki/AlignIO} and by
+a whole chapter in our tutorial:
+ - U{http://biopython.org/DIST/docs/tutorial/Tutorial.html}
+ - U{http://biopython.org/DIST/docs/tutorial/Tutorial.pdf}  
+
 Input
 =====
 For the typical special case when your file or handle contains one and only
@@ -19,52 +24,39 @@ handle, format string and optional number of sequences per alignment.  It will
 return a single Alignment object (or raise an exception if there isn't just
 one alignment):
 
-    from Bio import AlignIO
-    handle = open("example.aln", "rU")
-    align = AlignIO.read(handle, "clustal")
-    handle.close()
-    print align
+    >>> from Bio import AlignIO
+    >>> handle = open("Phylip/interlaced.phy", "rU")
+    >>> align = AlignIO.read(handle, "phylip")
+    >>> handle.close()
+    >>> print align
+    SingleLetterAlphabet() alignment with 3 rows and 384 columns
+    -----MKVILLFVLAVFTVFVSS---------------RGIPPE...I-- CYS1_DICDI
+    MAHARVLLLALAVLATAAVAVASSSSFADSNPIRPVTDRAASTL...VAA ALEU_HORVU
+    ------MWATLPLLCAGAWLLGV--------PVCGAAELSVNSL...PLV CATH_HUMAN
 
 For the general case, when the handle could contain any number of alignments,
 use the function Bio.AlignIO.parse(...) which takes the same arguments, but
-returns an iterator giving Alignment objects.  For example, using the output
-from the EMBOSS water or needle pairwise alignment prorams:
-
-    from Bio import AlignIO
-    handle = open("example.txt", "rU")
-    for alignment in AlignIO.parse(handle, "emboss") :
-        print alignment
-
+returns an iterator giving Alignment objects (typically used in a for loop).
 If you want random access to the alignments by number, turn this into a list:
 
-    from Bio import AlignIO
-    handle = open("example.aln", "rU")
-    alignments = list(AlignIO.parse(handle, "clustal"))
-    print alignments[0]
+    >>> from Bio import AlignIO
+    >>> handle = open("Emboss/needle.txt", "rU")
+    >>> alignments = list(AlignIO.parse(handle, "emboss"))
+    >>> print alignments[2]
+    SingleLetterAlphabet() alignment with 2 rows and 120 columns
+    -KILIVDDQYGIRILLNEVFNKEGYQTFQAANGLQALDIVTKER...--- ref_rec
+    LHIVVVDDDPGTCVYIESVFAELGHTCKSFVRPEAAEEYILTHP...HKE gi|94967506|receiver
 
 Most alignment file formats can be concatenated so as to hold as many
 different multiple sequence alignments as possible.  One common example
 is the output of the tool seqboot in the PHLYIP suite.  Sometimes there
 can be a file header and footer, as seen in the EMBOSS alignment output.
 
-There is an optional argument for the number of sequences per alignment which
-is usually only needed with the alignments stored in the FASTA format.
-Without this information, there is no clear way to tell if you have say a
-single alignment of 20 sequences, or four alignments of 5 sequences.  e.g.
-
-    from Bio import AlignIO
-    handle = open("example.faa", "rU")
-    for alignment in AlignIO.parse(handle, "fasta", seq_count=5) :
-        print alignment
-
-The above code would split up the FASTA files, and try and batch every five
-sequences into an alignment.
-
 Output
 ======
 Use the function Bio.AlignIO.write(...), which takes a complete set of
 Alignment objects (either as a list, or an iterator), an output file handle
-and of course the file format.
+and of course the file format::
 
     from Bio import AlignIO
     alignments = ...
@@ -78,26 +70,37 @@ like PHYLIP where multiple alignments are stored sequentially (with no file
 header and footer), then multiple calls to the write function should work as
 expected.
 
+Conversion
+==========
+The Bio.AlignIO.convert(...) function allows an easy interface for simple
+alignnment file format conversions. Additionally, it may use file format
+specific optimisations so this should be the fastest way too.
+
+In general however, you can combine the Bio.AlignIO.parse(...) function with
+the Bio.AlignIO.write(...) function for sequence file conversion. Using
+generator expressions provides a memory efficient way to perform filtering or
+other extra operations as part of the process.
+
 File Formats
 ============
 When specifying the file format, use lowercase strings.  The same format
 names are also used in Bio.SeqIO and include the following:
 
-clustal   - Ouput from Clustal W or X, see also the module Bio.Clustalw
-            which can be used to run the command line tool from Biopython.
-emboss    - The "pairs" and "simple" alignment format from the EMBOSS tools.
-fasta     - The generic sequence file format where each record starts with a
-            identifer line starting with a ">" character, followed by lines
-            of sequence.
-fasta-m10 - For the pairswise alignments output by Bill Pearson's FASTA
-            tools when used with the -m 10 command line option for machine
-            readable output.
-ig        - The IntelliGenetics file format, apparently the same as the
-            MASE alignment format.
-nexus     - Output from NEXUS, see also the module Bio.Nexus which can also
-            read any phylogenetic trees in these files.
-phylip    - Used by the PHLIP tools.
-stockholm - A richly annotated alignment file format used by PFAM.
+ - clustal   - Ouput from Clustal W or X, see also the module Bio.Clustalw
+               which can be used to run the command line tool from Biopython.
+ - emboss    - EMBOSS tools' "pairs" and "simple" alignment formats.
+ - fasta     - The generic sequence file format where each record starts with
+               an identifer line starting with a ">" character, followed by
+               lines of sequence.
+ - fasta-m10 - For the pairswise alignments output by Bill Pearson's FASTA
+               tools when used with the -m 10 command line option for machine
+               readable output.
+ - ig        - The IntelliGenetics file format, apparently the same as the
+               MASE alignment format.
+ - nexus     - Output from NEXUS, see also the module Bio.Nexus which can also
+               read any phylogenetic trees in these files.
+ - phylip    - Used by the PHLIP tools.
+ - stockholm - A richly annotated alignment file format used by PFAM.
 
 Note that while Bio.AlignIO can read all the above file formats, it cannot
 write to all of them.
@@ -105,15 +108,8 @@ write to all of them.
 You can also use any file format supported by Bio.SeqIO, such as "fasta" or
 "ig" (which are listed above), PROVIDED the sequences in your file are all the
 same length.
-
-Further Information
-===================
-See the wiki page http://biopython.org/wiki/AlignIO and also the Bio.AlignIO
-chapter in the Biopython Tutorial and Cookbook which is also available online:
-
-http://biopython.org/DIST/docs/tutorial/Tutorial.html
-http://biopython.org/DIST/docs/tutorial/Tutorial.pdf
 """
+__docformat__ = "epytext en" #not just plaintext
 
 #TODO
 # - define policy on reading aligned sequences with gaps in
@@ -164,12 +160,13 @@ _FormatToWriter ={#"fasta" is done via Bio.SeqIO
                   "clustal" : ClustalIO.ClustalWriter,
                   }
 
-def write(alignments, handle, format) :
+def write(alignments, handle, format):
     """Write complete set of alignments to a file.
 
-    sequences - A list (or iterator) of Alignment objects
-    handle    - File handle object to write to
-    format    - lower case string describing the file format to write.
+    Arguments:
+     - sequences - A list (or iterator) of Alignment objects
+     - handle    - File handle object to write to
+     - format    - lower case string describing the file format to write.
 
     You should close the handle after calling this function.
 
@@ -178,48 +175,53 @@ def write(alignments, handle, format) :
     from Bio import SeqIO
 
     #Try and give helpful error messages:
-    if isinstance(handle, basestring) :
+    if isinstance(handle, basestring):
         raise TypeError("Need a file handle, not a string (i.e. not a filename)")
-    if not isinstance(format, basestring) :
+    if not isinstance(format, basestring):
         raise TypeError("Need a string for the file format (lower case)")
-    if not format :
+    if not format:
         raise ValueError("Format required (lower case string)")
-    if format != format.lower() :
+    if format != format.lower():
         raise ValueError("Format string '%s' should be lower case" % format)
-    if isinstance(alignments, Alignment) :
+    if isinstance(alignments, Alignment):
         raise TypeError("Need an Alignment list/iterator, not just a single Alignment")
 
     #Map the file format to a writer class
-    if format in _FormatToIterator :
+    if format in _FormatToIterator:
         writer_class = _FormatToWriter[format]
         count = writer_class(handle).write_file(alignments)
-    elif format in SeqIO._FormatToWriter :
+    elif format in SeqIO._FormatToWriter:
         #Exploit the existing SeqIO parser to the dirty work!
         #TODO - Can we make one call to SeqIO.write() and count the alignments?
         count = 0
-        for alignment in alignments :
+        for alignment in alignments:
+            if not isinstance(alignment, Alignment):
+                raise TypeError("Expect a list or iterator of Alignment objects.")
             SeqIO.write(alignment, handle, format)
             count += 1
-    elif format in _FormatToIterator or format in SeqIO._FormatToIterator :
+    elif format in _FormatToIterator or format in SeqIO._FormatToIterator:
         raise ValueError("Reading format '%s' is supported, but not writing" \
                          % format)
-    else :
+    else:
         raise ValueError("Unknown format '%s'" % format)
 
-    assert isinstance(count, int), "Internal error - the underlying writer " \
-           + " should have returned the alignment count, not %s" % repr(count)
+    assert isinstance(count, int), "Internal error - the underlying %s " \
+           "writer should have returned the alignment count, not %s" \
+           % (format, repr(count))
     return count
 
 #This is a generator function!
-def _SeqIO_to_alignment_iterator(handle, format, alphabet=None, seq_count=None) :
+def _SeqIO_to_alignment_iterator(handle, format, alphabet=None, seq_count=None):
     """Uses Bio.SeqIO to create an Alignment iterator (PRIVATE).
 
-    handle   - handle to the file.
-    format   - string describing the file format.
-    alphabet - optional Alphabet object, useful when the sequence type cannot
-               be automatically inferred from the file itself (e.g. fasta)
-    seq_count- Optional integer, number of sequences expected in
-               each alignment.  Recommended for fasta format files.
+    Arguments:
+     - handle    - handle to the file.
+     - format    - string describing the file format.
+     - alphabet  - optional Alphabet object, useful when the sequence type
+                   cannot be automatically inferred from the file itself
+                   (e.g. fasta, phylip, clustal)
+     - seq_count - Optional integer, number of sequences expected in each
+                   alignment.  Recommended for fasta format files.
 
     If count is omitted (default) then all the sequences in
     the file are combined into a single Alignment.
@@ -227,41 +229,41 @@ def _SeqIO_to_alignment_iterator(handle, format, alphabet=None, seq_count=None)
     from Bio import SeqIO
     assert format in SeqIO._FormatToIterator
 
-    if seq_count :
+    if seq_count:
         #Use the count to split the records into batches.
         seq_record_iterator = SeqIO.parse(handle, format, alphabet)
 
         records = []
-        for record in seq_record_iterator :
+        for record in seq_record_iterator:
             records.append(record)
-            if len(records) == seq_count :
+            if len(records) == seq_count:
                 yield SeqIO.to_alignment(records)
                 records = []
-        if len(records) > 0 :
+        if len(records) > 0:
             raise ValueError("Check seq_count argument, not enough sequences?")
-    else :
+    else:
         #Must assume that there is a single alignment using all
         #the SeqRecord objects:
         records = list(SeqIO.parse(handle, format, alphabet))
-        if records :
+        if records:
             yield SeqIO.to_alignment(records)
-        else :
+        else:
             #No alignment found!
             pass
 
-def _force_alphabet(alignment_iterator, alphabet) :
+def _force_alphabet(alignment_iterator, alphabet):
      """Iterate over alignments, over-riding the alphabet (PRIVATE)."""
      #Assume the alphabet argument has been pre-validated
      given_base_class = _get_base_alphabet(alphabet).__class__
-     for align in alignment_iterator :
+     for align in alignment_iterator:
          if not isinstance(_get_base_alphabet(align._alphabet),
-                           given_base_class) :
+                           given_base_class):
              raise ValueError("Specified alphabet %s clashes with "\
                               "that determined from the file, %s" \
                               % (repr(alphabet), repr(align._alphabet)))
-         for record in align :
+         for record in align:
              if not isinstance(_get_base_alphabet(record.seq.alphabet),
-                               given_base_class) :
+                               given_base_class):
                  raise ValueError("Specified alphabet %s clashes with "\
                                   "that determined from the file, %s" \
                             % (repr(alphabet), repr(record.seq.alphabet)))
@@ -269,22 +271,24 @@ def _force_alphabet(alignment_iterator, alphabet) :
          align._alphabet = alphabet
          yield align
     
-def parse(handle, format, seq_count=None, alphabet=None) :
+def parse(handle, format, seq_count=None, alphabet=None):
     """Turns a sequence file into an iterator returning Alignment objects.
 
-    handle   - handle to the file.
-    format   - string describing the file format.
-    alphabet - optional Alphabet object, useful when the sequence type cannot
-               be automatically inferred from the file itself (e.g. phylip)
-    seq_count- Optional integer, number of sequences expected in
-               each alignment.  Recommended for fasta format files.
+    Arguments:
+     - handle    - handle to the file.
+     - format    - string describing the file format.
+     - alphabet  - optional Alphabet object, useful when the sequence type
+                   cannot be automatically inferred from the file itself
+                   (e.g. fasta, phylip, clustal)
+     - seq_count - Optional integer, number of sequences expected in each
+                   alignment.  Recommended for fasta format files.
 
     If you have the file name in a string 'filename', use:
 
     >>> from Bio import AlignIO
     >>> filename = "Emboss/needle.txt"
     >>> format = "emboss"
-    >>> for alignment in AlignIO.parse(open(filename,"rU"), format) :
+    >>> for alignment in AlignIO.parse(open(filename,"rU"), format):
     ...     print "Alignment of length", alignment.get_alignment_length()
     Alignment of length 124
     Alignment of length 119
@@ -303,47 +307,51 @@ def parse(handle, format, seq_count=None, alphabet=None) :
     from Bio import SeqIO
 
     #Try and give helpful error messages:
-    if isinstance(handle, basestring) :
+    if isinstance(handle, basestring):
         raise TypeError("Need a file handle, not a string (i.e. not a filename)")
-    if not isinstance(format, basestring) :
+    if not isinstance(format, basestring):
         raise TypeError("Need a string for the file format (lower case)")
-    if not format :
+    if not format:
         raise ValueError("Format required (lower case string)")
-    if format != format.lower() :
+    if format != format.lower():
         raise ValueError("Format string '%s' should be lower case" % format)
     if alphabet is not None and not (isinstance(alphabet, Alphabet) or \
-                                     isinstance(alphabet, AlphabetEncoder)) :
+                                     isinstance(alphabet, AlphabetEncoder)):
         raise ValueError("Invalid alphabet, %s" % repr(alphabet))
+    if seq_count is not None and not isinstance(seq_count, int):
+        raise TypeError("Need integer for seq_count (sequences per alignment)")
 
     #Map the file format to a sequence iterator:
-    if format in _FormatToIterator :
+    if format in _FormatToIterator:
         iterator_generator = _FormatToIterator[format]
         if alphabet is None : 
             return iterator_generator(handle, seq_count)
-        try :
+        try:
             #Initially assume the optional alphabet argument is supported
             return iterator_generator(handle, seq_count, alphabet=alphabet)
-        except TypeError :
+        except TypeError:
             #It isn't supported.
             return _force_alphabet(iterator_generator(handle, seq_count), alphabet)
 
-    elif format in SeqIO._FormatToIterator :
+    elif format in SeqIO._FormatToIterator:
         #Exploit the existing SeqIO parser to the dirty work!
         return _SeqIO_to_alignment_iterator(handle, format,
                                             alphabet=alphabet,
                                             seq_count=seq_count)
-    else :
+    else:
         raise ValueError("Unknown format '%s'" % format)
 
-def read(handle, format, seq_count=None, alphabet=None) :
+def read(handle, format, seq_count=None, alphabet=None):
     """Turns an alignment file into a single Alignment object.
 
-    handle   - handle to the file.
-    format   - string describing the file format.
-    alphabet - optional Alphabet object, useful when the sequence type cannot
-               be automatically inferred from the file itself (e.g. phylip)
-    seq_count- Optional interger, number of sequences expected in
-               the alignment to check you got what you expected.
+    Arguments:
+     - handle    - handle to the file.
+     - format    - string describing the file format.
+     - alphabet  - optional Alphabet object, useful when the sequence type
+                   cannot be automatically inferred from the file itself
+                   (e.g. fasta, phylip, clustal)
+     - seq_count - Optional integer, number of sequences expected in each
+                   alignment.  Recommended for fasta format files.
 
     If the handle contains no alignments, or more than one alignment,
     an exception is raised.  For example, using a PFAM/Stockholm file
@@ -358,6 +366,15 @@ def read(handle, format, seq_count=None, alphabet=None) :
 
     If however you want the first alignment from a file containing
     multiple alignments this function would raise an exception.
+
+    >>> from Bio import AlignIO
+    >>> filename = "Emboss/needle.txt"
+    >>> format = "emboss"
+    >>> alignment = AlignIO.read(open(filename, "rU"), format)
+    Traceback (most recent call last):
+        ...
+    ValueError: More than one record found in handle
+
     Instead use:
 
     >>> from Bio import AlignIO
@@ -367,26 +384,64 @@ def read(handle, format, seq_count=None, alphabet=None) :
     >>> print "First alignment has length", alignment.get_alignment_length()
     First alignment has length 124
 
-    Use the Bio.AlignIO.parse() function if you want to read multiple
+    You must use the Bio.AlignIO.parse() function if you want to read multiple
     records from the handle.
     """
     iterator = parse(handle, format, seq_count, alphabet)
-    try :
+    try:
         first = iterator.next()
-    except StopIteration :
+    except StopIteration:
         first = None
-    if first is None :
+    if first is None:
         raise ValueError("No records found in handle")
-    try :
+    try:
         second = iterator.next()
-    except StopIteration :
+    except StopIteration:
         second = None
-    if second is not None :
+    if second is not None:
         raise ValueError("More than one record found in handle")
-    if seq_count :
+    if seq_count:
         assert len(first.get_all_seqs())==seq_count
     return first
 
+def convert(in_file, in_format, out_file, out_format, alphabet=None):
+    """Convert between two alignment files, returns number of alignments.
+
+     - in_file - an input handle or filename
+     - in_format - input file format, lower case string
+     - output - an output handle or filename
+     - out_file - output file format, lower case string
+     - alphabet - optional alphabet to assume
+
+    NOTE - If you provide an output filename, it will be opened which will
+    overwrite any existing file without warning. This may happen if even the
+    conversion is aborted (e.g. an invalid out_format name is given).
+    """
+    #TODO - Add optimised versions of important conversions
+    #For now just off load the work to SeqIO parse/write    
+    if isinstance(in_file, basestring):
+        in_handle = open(in_file, "rU")
+        in_close = True
+    else:
+        in_handle = in_file
+        in_close = False
+    #This will check the arguments and issue error messages,
+    alignments = parse(in_handle, in_format, None, alphabet)
+    #Don't open the output file until we've checked the input is OK:
+    if isinstance(out_file, basestring):
+        out_handle = open(out_file, "w")
+        out_close = True
+    else:
+        out_handle = out_file
+        out_close = False
+    #This will check the arguments and issue error messages,
+    #after we have opened the file which is a shame.
+    count = write(alignments, out_handle, out_format)
+    #Must now close any handles we opened
+    if in_close : in_handle.close()
+    if out_close : out_handle.close()
+    return count
+
 def _test():
     """Run the Bio.AlignIO module's doctests.
 
@@ -395,7 +450,7 @@ def _test():
     """
     import doctest
     import os
-    if os.path.isdir(os.path.join("..","..","Tests")) :
+    if os.path.isdir(os.path.join("..","..","Tests")):
         print "Runing doctests..."
         cur_dir = os.path.abspath(os.curdir)
         os.chdir(os.path.join("..","..","Tests"))
@@ -404,5 +459,5 @@ def _test():
         del cur_dir
         print "Done"
         
-if __name__ == "__main__" :
+if __name__ == "__main__":
     _test()
