@@ -27,10 +27,6 @@
  * 
  */
 
-#ifndef CALL 
-# define CALL
-#endif
-
 #ifndef min
 #define min(x, y)	((x) < (y) ? (x) : (y))
 #endif
@@ -42,13 +38,13 @@
 #  include <windows.h>
 #endif
 
-#define CLUSTERVERSION "1.42"
+#define CLUSTERVERSION "1.47"
 
 /* Chapter 2 */
-double CALL clusterdistance (int nrows, int ncolumns, double** data, int** mask,
+double clusterdistance (int nrows, int ncolumns, double** data, int** mask,
   double weight[], int n1, int n2, int index1[], int index2[], char dist,
   char method, int transpose);
-double** CALL distancematrix (int ngenes, int ndata, double** data,
+double** distancematrix (int ngenes, int ndata, double** data,
   int** mask, double* weight, char dist, int transpose);
 
 /* Chapter 3 */
@@ -57,10 +53,10 @@ int getclustercentroids(int nclusters, int nrows, int ncolumns,
   int transpose, char method);
 void getclustermedoids(int nclusters, int nelements, double** distance,
   int clusterid[], int centroids[], double errors[]);
-void CALL kcluster (int nclusters, int ngenes, int ndata, double** data,
+void kcluster (int nclusters, int ngenes, int ndata, double** data,
   int** mask, double weight[], int transpose, int npass, char method, char dist,
   int clusterid[], double* error, int* ifound);
-void CALL kmedoids (int nclusters, int nelements, double** distance,
+void kmedoids (int nclusters, int nelements, double** distance,
   int npass, int clusterid[], double* error, int* ifound);
 
 /* Chapter 4 */
@@ -75,23 +71,23 @@ typedef struct {int left; int right; double distance;} Node;
  * between the two subnodes that were joined.
  */
 
-Node* CALL treecluster (int nrows, int ncolumns, double** data, int** mask,
+Node* treecluster (int nrows, int ncolumns, double** data, int** mask,
   double weight[], int transpose, char dist, char method, double** distmatrix);
 void cuttree (int nelements, Node* tree, int nclusters, int clusterid[]);
 
 /* Chapter 5 */
-void CALL somcluster (int nrows, int ncolumns, double** data, int** mask,
+void somcluster (int nrows, int ncolumns, double** data, int** mask,
   const double weight[], int transpose, int nxnodes, int nynodes,
   double inittau, int niter, char dist, double*** celldata,
   int clusterid[][2]);
 
 /* Chapter 6 */
-void CALL svd(int m, int n, double** u, double w[], double** v, int* ierr);
+void svd(int m, int n, double** u, double w[], double** v, int* ierr);
 
 /* Utility routines, currently undocumented */
-void CALL sort(int n, const double data[], int index[]);
-double CALL mean(int n, double x[]);
-double CALL median (int n, double x[]);
+void sort(int n, const double data[], int index[]);
+double mean(int n, double x[]);
+double median (int n, double x[]);
 
 double* calculate_weights(int nrows, int ncolumns, double** data, int** mask,
   double weights[], int transpose, char dist, double cutoff, double exponent);
