@@ -1,10 +1,14 @@
-#TODO - Remove this work around once we drop python 2.3 support
-try:
-   set = set
-except NameError:
-   from sets import Set as set
+# This code is part of the Biopython distribution and governed by its
+# license.  Please see the LICENSE file that should have been included
+# as part of this package.
+"""Codon tables based on those from the NCBI.
+
+These tables are based on parsing the NCBI file:
+ftp://ftp.ncbi.nih.gov/entrez/misc/data/gc.prt
+
+Last updated for Version 3.9
+"""
 
-import string
 from Bio import Alphabet
 from Bio.Alphabet import IUPAC
 from Bio.Data import IUPACData
@@ -49,7 +53,7 @@ class CodonTable:
         self.start_codons = start_codons
         self.stop_codons = stop_codons
 
-    def __str__(self) :
+    def __str__(self):
         """Returns a simple text representation of the codon table
 
         e.g.
@@ -57,20 +61,20 @@ class CodonTable:
         >>> print Bio.Data.CodonTable.standard_dna_table
         >>> print Bio.Data.CodonTable.generic_by_id[1]"""
 
-        if self.id :
+        if self.id:
             answer = "Table %i" % self.id
-        else :
+        else:
             answer = "Table ID unknown"
-        if self.names :
+        if self.names:
             answer += " " + ", ".join(filter(None, self.names))
 
         #Use the main four letters (and the conventional ordering)
         #even for ambiguous tables
         letters = self.nucleotide_alphabet.letters
         if isinstance(self.nucleotide_alphabet, Alphabet.DNAAlphabet) \
-        or (letters is not None and "T" in letters) :
+        or (letters is not None and "T" in letters):
             letters = "TCAG"
-        else :
+        else:
             #Should be either RNA or generic nucleotides,
             #e.g. Bio.Data.CodonTable.generic_by_id[1]
             letters = "UCAG"
@@ -81,24 +85,24 @@ class CodonTable:
             ) + "|"
         answer=answer + "\n--+" \
                + "+".join(["---------" for c2 in letters]) + "+--"
-        for c1 in letters :
-            for c3 in letters :
+        for c1 in letters:
+            for c3 in letters:
                 line = c1 + " |"
-                for c2 in letters :
+                for c2 in letters:
                     codon = c1+c2+c3
                     line = line + " %s" % codon
-                    if codon in self.stop_codons :
+                    if codon in self.stop_codons:
                         line = line + " Stop|"
-                    else :
-                        try :
+                    else:
+                        try:
                             amino = self.forward_table[codon]
-                        except KeyError :
+                        except KeyError:
                             amino = "?"
-                        except TranslationError :
+                        except TranslationError:
                             amino = "?"
-                        if codon in self.start_codons :
+                        if codon in self.start_codons:
                             line = line + " %s(s)|" % amino
-                        else :
+                        else:
                             line = line + " %s   |" % amino
                 line = line + " " + c3
                 answer = answer + "\n"+ line 
@@ -141,7 +145,9 @@ class NCBICodonTableRNA(NCBICodonTable):
 
 def register_ncbi_table(name, alt_name, id,
                         table, start_codons, stop_codons):
-    names = string.split(name, "; ")
+    #In most cases names are divided by "; ", however there is also
+    #'Bacterial and Plant Plastid' (which used to be just 'Bacterial')
+    names = [x.strip() for x in name.replace(" and ","; ").split("; ")]
     
     dna = NCBICodonTableDNA(id, names + [alt_name], table, start_codons,
                             stop_codons)
@@ -191,7 +197,7 @@ def register_ncbi_table(name, alt_name, id,
         generic_by_name[name] = generic
 
 ### These tables created from the data file
-###  ftp://ncbi.nlm.nih.gov/entrez/misc/data/gc.prt
+###  ftp://ftp.ncbi.nih.gov/entrez/misc/data/gc.prt
 ### using the following:
 ##import re
 ##for line in open("gc.prt").readlines():
@@ -221,7 +227,7 @@ def register_ncbi_table(name, alt_name, id,
 ##        if len(names) == 1:
 ##            names.append(None)
 ##        print "register_ncbi_table(name = %s," % repr(names[0])
-##        print "                    alt_name = %s, id = %d", % \
+##        print "                    alt_name = %s, id = %d," % \
 ##              (repr(names[1]), id)
 ##        print "                    table = {"
 ##        s = "    "
@@ -318,7 +324,7 @@ register_ncbi_table(name = 'Yeast Mitochondrial',
      'GAC': 'D', 'GAA': 'E', 'GAG': 'E', 'GGT': 'G', 'GGC': 'G',
      'GGA': 'G', 'GGG': 'G', },
                     stop_codons = [ 'TAA', 'TAG', ],
-                    start_codons = [ 'ATG', ]
+                    start_codons = [ 'ATA', 'ATG', ]
                     )
 register_ncbi_table(name = 'Mold Mitochondrial; Protozoan Mitochondrial; Coelenterate Mitochondrial; Mycoplasma; Spiroplasma',
                     alt_name = 'SGC3', id = 4,
@@ -379,7 +385,7 @@ register_ncbi_table(name = 'Ciliate Nuclear; Dasycladacean Nuclear; Hexamita Nuc
                     stop_codons = [ 'TGA', ],
                     start_codons = [ 'ATG', ]
                     )
-register_ncbi_table(name = 'Echinoderm Mitochondrial',
+register_ncbi_table(name = 'Echinoderm Mitochondrial; Flatworm Mitochondrial',
                     alt_name = 'SGC8', id = 9,
                     table = {
      'TTT': 'F', 'TTC': 'F', 'TTA': 'L', 'TTG': 'L', 'TCT': 'S',
@@ -396,7 +402,7 @@ register_ncbi_table(name = 'Echinoderm Mitochondrial',
      'GAC': 'D', 'GAA': 'E', 'GAG': 'E', 'GGT': 'G', 'GGC': 'G',
      'GGA': 'G', 'GGG': 'G', },
                     stop_codons = [ 'TAA', 'TAG', ],
-                    start_codons = [ 'ATG', ]
+                    start_codons = [ 'ATG', 'GTG', ]
                     )
 register_ncbi_table(name = 'Euplotid Nuclear',
                     alt_name = 'SGC9', id = 10,
@@ -417,7 +423,7 @@ register_ncbi_table(name = 'Euplotid Nuclear',
                     stop_codons = [ 'TAA', 'TAG', ],
                     start_codons = [ 'ATG', ]
                     )
-register_ncbi_table(name = 'Bacterial',
+register_ncbi_table(name = 'Bacterial and Plant Plastid',
                     alt_name = None, id = 11,
                     table = {
      'TTT': 'F', 'TTC': 'F', 'TTA': 'L', 'TTG': 'L', 'TCT': 'S',
@@ -473,9 +479,9 @@ register_ncbi_table(name = 'Ascidian Mitochondrial',
      'GAC': 'D', 'GAA': 'E', 'GAG': 'E', 'GGT': 'G', 'GGC': 'G',
      'GGA': 'G', 'GGG': 'G', },
                     stop_codons = [ 'TAA', 'TAG', ],
-                    start_codons = [ 'ATG', ]
+                    start_codons = [ 'TTG', 'ATA', 'ATG', 'GTG', ]
                     )
-register_ncbi_table(name = 'Flatworm Mitochondrial',
+register_ncbi_table(name = 'Alternative Flatworm Mitochondrial',
                     alt_name = None, id = 14,
                     table = {
      'TTT': 'F', 'TTC': 'F', 'TTA': 'L', 'TTG': 'L', 'TCT': 'S',
@@ -513,6 +519,82 @@ register_ncbi_table(name = 'Blepharisma Macronuclear',
                     stop_codons = [ 'TAA', 'TGA', ],
                     start_codons = [ 'ATG', ]
                     )
+register_ncbi_table(name = 'Chlorophycean Mitochondrial',
+                    alt_name = None, id = 16,
+                    table = {
+     'TTT': 'F', 'TTC': 'F', 'TTA': 'L', 'TTG': 'L', 'TCT': 'S',
+     'TCC': 'S', 'TCA': 'S', 'TCG': 'S', 'TAT': 'Y', 'TAC': 'Y',
+     'TAG': 'L', 'TGT': 'C', 'TGC': 'C', 'TGG': 'W', 'CTT': 'L',
+     'CTC': 'L', 'CTA': 'L', 'CTG': 'L', 'CCT': 'P', 'CCC': 'P',
+     'CCA': 'P', 'CCG': 'P', 'CAT': 'H', 'CAC': 'H', 'CAA': 'Q',
+     'CAG': 'Q', 'CGT': 'R', 'CGC': 'R', 'CGA': 'R', 'CGG': 'R',
+     'ATT': 'I', 'ATC': 'I', 'ATA': 'I', 'ATG': 'M', 'ACT': 'T',
+     'ACC': 'T', 'ACA': 'T', 'ACG': 'T', 'AAT': 'N', 'AAC': 'N',
+     'AAA': 'K', 'AAG': 'K', 'AGT': 'S', 'AGC': 'S', 'AGA': 'R',
+     'AGG': 'R', 'GTT': 'V', 'GTC': 'V', 'GTA': 'V', 'GTG': 'V',
+     'GCT': 'A', 'GCC': 'A', 'GCA': 'A', 'GCG': 'A', 'GAT': 'D',
+     'GAC': 'D', 'GAA': 'E', 'GAG': 'E', 'GGT': 'G', 'GGC': 'G',
+     'GGA': 'G', 'GGG': 'G', },
+                    stop_codons = [ 'TAA', 'TGA', ],
+                    start_codons = [ 'ATG', ]
+                    )
+register_ncbi_table(name = 'Trematode Mitochondrial',
+                    alt_name = None, id = 21,
+                    table = {
+     'TTT': 'F', 'TTC': 'F', 'TTA': 'L', 'TTG': 'L', 'TCT': 'S',
+     'TCC': 'S', 'TCA': 'S', 'TCG': 'S', 'TAT': 'Y', 'TAC': 'Y',
+     'TGT': 'C', 'TGC': 'C', 'TGA': 'W', 'TGG': 'W', 'CTT': 'L',
+     'CTC': 'L', 'CTA': 'L', 'CTG': 'L', 'CCT': 'P', 'CCC': 'P',
+     'CCA': 'P', 'CCG': 'P', 'CAT': 'H', 'CAC': 'H', 'CAA': 'Q',
+     'CAG': 'Q', 'CGT': 'R', 'CGC': 'R', 'CGA': 'R', 'CGG': 'R',
+     'ATT': 'I', 'ATC': 'I', 'ATA': 'M', 'ATG': 'M', 'ACT': 'T',
+     'ACC': 'T', 'ACA': 'T', 'ACG': 'T', 'AAT': 'N', 'AAC': 'N',
+     'AAA': 'N', 'AAG': 'K', 'AGT': 'S', 'AGC': 'S', 'AGA': 'S',
+     'AGG': 'S', 'GTT': 'V', 'GTC': 'V', 'GTA': 'V', 'GTG': 'V',
+     'GCT': 'A', 'GCC': 'A', 'GCA': 'A', 'GCG': 'A', 'GAT': 'D',
+     'GAC': 'D', 'GAA': 'E', 'GAG': 'E', 'GGT': 'G', 'GGC': 'G',
+     'GGA': 'G', 'GGG': 'G', },
+                    stop_codons = [ 'TAA', 'TAG', ],
+                    start_codons = [ 'ATG', 'GTG', ]
+                    )
+register_ncbi_table(name = 'Scenedesmus obliquus Mitochondrial',
+                    alt_name = None, id = 22,
+                    table = {
+     'TTT': 'F', 'TTC': 'F', 'TTA': 'L', 'TTG': 'L', 'TCT': 'S',
+     'TCC': 'S', 'TCG': 'S', 'TAT': 'Y', 'TAC': 'Y', 'TAG': 'L',
+     'TGT': 'C', 'TGC': 'C', 'TGG': 'W', 'CTT': 'L', 'CTC': 'L',
+     'CTA': 'L', 'CTG': 'L', 'CCT': 'P', 'CCC': 'P', 'CCA': 'P',
+     'CCG': 'P', 'CAT': 'H', 'CAC': 'H', 'CAA': 'Q', 'CAG': 'Q',
+     'CGT': 'R', 'CGC': 'R', 'CGA': 'R', 'CGG': 'R', 'ATT': 'I',
+     'ATC': 'I', 'ATA': 'I', 'ATG': 'M', 'ACT': 'T', 'ACC': 'T',
+     'ACA': 'T', 'ACG': 'T', 'AAT': 'N', 'AAC': 'N', 'AAA': 'K',
+     'AAG': 'K', 'AGT': 'S', 'AGC': 'S', 'AGA': 'R', 'AGG': 'R',
+     'GTT': 'V', 'GTC': 'V', 'GTA': 'V', 'GTG': 'V', 'GCT': 'A',
+     'GCC': 'A', 'GCA': 'A', 'GCG': 'A', 'GAT': 'D', 'GAC': 'D',
+     'GAA': 'E', 'GAG': 'E', 'GGT': 'G', 'GGC': 'G', 'GGA': 'G',
+     'GGG': 'G', },
+                    stop_codons = [ 'TCA', 'TAA', 'TGA', ],
+                    start_codons = [ 'ATG', ]
+                    )
+register_ncbi_table(name = 'Thraustochytrium Mitochondrial',
+                    alt_name = None, id = 23,
+                    table = {
+     'TTT': 'F', 'TTC': 'F', 'TTG': 'L', 'TCT': 'S', 'TCC': 'S',
+     'TCA': 'S', 'TCG': 'S', 'TAT': 'Y', 'TAC': 'Y', 'TGT': 'C',
+     'TGC': 'C', 'TGG': 'W', 'CTT': 'L', 'CTC': 'L', 'CTA': 'L',
+     'CTG': 'L', 'CCT': 'P', 'CCC': 'P', 'CCA': 'P', 'CCG': 'P',
+     'CAT': 'H', 'CAC': 'H', 'CAA': 'Q', 'CAG': 'Q', 'CGT': 'R',
+     'CGC': 'R', 'CGA': 'R', 'CGG': 'R', 'ATT': 'I', 'ATC': 'I',
+     'ATA': 'I', 'ATG': 'M', 'ACT': 'T', 'ACC': 'T', 'ACA': 'T',
+     'ACG': 'T', 'AAT': 'N', 'AAC': 'N', 'AAA': 'K', 'AAG': 'K',
+     'AGT': 'S', 'AGC': 'S', 'AGA': 'R', 'AGG': 'R', 'GTT': 'V',
+     'GTC': 'V', 'GTA': 'V', 'GTG': 'V', 'GCT': 'A', 'GCC': 'A',
+     'GCA': 'A', 'GCG': 'A', 'GAT': 'D', 'GAC': 'D', 'GAA': 'E',
+     'GAG': 'E', 'GGT': 'G', 'GGC': 'G', 'GGA': 'G', 'GGG': 'G', },
+                    stop_codons = [ 'TTA', 'TAA', 'TAG', 'TGA', ],
+                    start_codons = [ 'ATT', 'ATG', 'GTG', ]
+                    )
+
 
 #########  Deal with ambiguous forward translations
 
@@ -584,36 +666,41 @@ def list_ambiguous_codons(codons, ambiguous_nucleotide_values):
     #Note ambiguous_nucleotide_values['R'] = 'AG' (etc)
     #This will generate things like 'TRR' from ['TAG', 'TGA'], which
     #we don't want to include:
-    c1_list = [letter for (letter, meanings) \
+    c1_list = sorted(letter for (letter, meanings) \
                in ambiguous_nucleotide_values.iteritems() \
-               if set([codon[0] for codon in codons]).issuperset(set(meanings))]
-    c2_list = [letter for (letter, meanings) \
+               if set([codon[0] for codon in codons]).issuperset(set(meanings)))
+    c2_list = sorted(letter for (letter, meanings) \
                in ambiguous_nucleotide_values.iteritems() \
-               if set([codon[1] for codon in codons]).issuperset(set(meanings))]
-    c3_list = [letter for (letter, meanings) \
+               if set([codon[1] for codon in codons]).issuperset(set(meanings)))
+    c3_list = sorted(letter for (letter, meanings) \
                in ambiguous_nucleotide_values.iteritems() \
-               if set([codon[2] for codon in codons]).issuperset(set(meanings))]
-    set2 = set([codon[1] for codon in codons])
-    set3 = set([codon[2] for codon in codons])
-    candidates = set([c1+c2+c3 for c1 in c1_list for c2 in c2_list for c3 in c3_list])
-    candidates.difference_update(codons)
+               if set([codon[2] for codon in codons]).issuperset(set(meanings)))
+    #candidates is a list (not a set) to preserve the iteration order
+    candidates = []
+    for c1 in c1_list:
+        for c2 in c2_list:
+            for c3 in c3_list:
+                codon = c1+c2+c3
+                if codon not in candidates and codon not in codons:
+                    candidates.append(codon)
     answer = codons[:] #copy
     #print "Have %i new candidates" % len(candidates)
-    for ambig_codon in candidates :
+    for ambig_codon in candidates:
         wanted = True
         #e.g. 'TRR' -> 'TAA', 'TAG', 'TGA', 'TGG'
         for codon in [c1+c2+c3 \
                       for c1 in ambiguous_nucleotide_values[ambig_codon[0]] \
                       for c2 in ambiguous_nucleotide_values[ambig_codon[1]] \
                       for c3 in ambiguous_nucleotide_values[ambig_codon[2]]]:
-            if codon not in codons :
+            if codon not in codons:
                 #This ambiguous codon can code for a non-stop, exclude it!
                 wanted=False
                 #print "Rejecting %s" % ambig_codon
                 continue
-        if wanted :
+        if wanted:
             answer.append(ambig_codon)
     return answer
+
 assert list_ambiguous_codons(['TGA', 'TAA'],IUPACData.ambiguous_dna_values) == ['TGA', 'TAA', 'TRA']
 assert list_ambiguous_codons(['TAG', 'TGA'],IUPACData.ambiguous_dna_values) == ['TAG', 'TGA']
 assert list_ambiguous_codons(['TAG', 'TAA'],IUPACData.ambiguous_dna_values) == ['TAG', 'TAA', 'TAR']
@@ -781,17 +868,19 @@ del _merged_values
 del key, val
 
 #Basic sanity test,
-for n in ambiguous_generic_by_id.keys() :
+for n in ambiguous_generic_by_id.keys():
     assert ambiguous_rna_by_id[n].forward_table["GUU"] == "V"
     assert ambiguous_rna_by_id[n].forward_table["GUN"] == "V"
-    assert ambiguous_rna_by_id[n].forward_table["UUN"] == "X" #F or L
+    if n != 23 :
+        #For table 23, UUN = F, L or stop.
+        assert ambiguous_rna_by_id[n].forward_table["UUN"] == "X" #F or L
     #R = A or G, so URR = UAA or UGA / TRA = TAA or TGA = stop codons
     if "UAA" in unambiguous_rna_by_id[n].stop_codons \
-    and "UGA" in unambiguous_rna_by_id[n].stop_codons :
-        try :
+    and "UGA" in unambiguous_rna_by_id[n].stop_codons:
+        try:
             print ambiguous_dna_by_id[n].forward_table["TRA"]
             assert False, "Should be a stop only"
-        except KeyError :
+        except KeyError:
             pass
         assert "URA" in ambiguous_generic_by_id[n].stop_codons
         assert "URA" in ambiguous_rna_by_id[n].stop_codons
@@ -800,4 +889,6 @@ for n in ambiguous_generic_by_id.keys() :
 del n
 assert ambiguous_generic_by_id[1].stop_codons == ambiguous_generic_by_name["Standard"].stop_codons
 assert ambiguous_generic_by_id[4].stop_codons == ambiguous_generic_by_name["SGC3"].stop_codons
+assert ambiguous_generic_by_id[11].stop_codons == ambiguous_generic_by_name["Bacterial"].stop_codons
+assert ambiguous_generic_by_id[11].stop_codons == ambiguous_generic_by_name["Plant Plastid"].stop_codons
 assert ambiguous_generic_by_id[15].stop_codons == ambiguous_generic_by_name['Blepharisma Macronuclear'].stop_codons
