@@ -5,9 +5,6 @@
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
 
-# make yield compatible with Python2.2
-from __future__ import generators
-
 """
 Bio.DocSQL: easy access to DB API databases.
 
@@ -25,8 +22,8 @@ Bio.DocSQL: easy access to DB API databases.
 CreatePeople(message=Success)
 """
 
-__version__ = "$Revision: 1.11 $"
-# $Source: /home/repository/biopython/biopython/Bio/DocSQL.py,v $
+__version__ = "$Revision: 1.13 $"
+# $Source: /home/bartek/cvs2bzr/biopython_fastimport/cvs_repo/biopython/Bio/DocSQL.py,v $
 
 import exceptions
 import sys
@@ -65,10 +62,9 @@ class QueryRow(list):
         _check_is_public(name)
         try:
             return self[self._names_hash[name]]
-        except KeyError:
-            raise AttributeError, "'%s' object has no attribute '%s'" % (self.__class__.__name__, name)
-        except AttributeError:
-            raise AttributeError, "'%s' object has no attribute '%s'" % (self.__class__.__name__, name)
+        except (KeyError, AttributeError):
+            raise AttributeError("'%s' object has no attribute '%s'" \
+                                 % (self.__class__.__name__, name))
 
     def __setattr__(self, name, value):
         try:
@@ -129,7 +125,7 @@ class QueryGeneric(Query):
 class IterationCursor(object):
     def __init__(self, query, connection=connection):
         if connection is None:
-            raise TypeError, "database connection is None"
+            raise TypeError("database connection is None")
         self.cursor = connection.cursor()
         self.row_class = query.row_class
         if query.diagnostics:
@@ -191,7 +187,7 @@ class Insert(Create):
             except AttributeError:
                 self.total_count = 0
             
-            raise MySQLdb.IntegrityError, self.error_message
+            raise MySQLdb.IntegrityError(self.error_message)
             
         self.id = self.cursor().insert_id()
         try:
