@@ -33,10 +33,6 @@ be used directly.
 # written solution, since the number of DTDs is rather large and their
 # contents may change over time. About half the code in this parser deals
 # wih parsing the DTD, and the other half with the XML itself.
-#
-# One type of query (EFetch on the Journals database) returns an XML without
-# an associated DTD. These files are handled using the hand-written
-# SerialSet.py
 
 
 import os.path
@@ -70,39 +66,107 @@ class StructureElement(dict):
         else:
             dict.__setitem__(self, key, value)
 
+
+class NotXMLError(ValueError):
+    def __str__(self):
+        return "Failed to parse the XML data. Please make sure that the input data are in XML format."
+
+
+class CorruptedXMLError(ValueError):
+    def __str__(self):
+        # This message can be changed once all XML data returned by EUtils
+        # start with the XML declaration
+        return "Failed to parse the XML data. Please make sure that the input data are in XML format, and that the data are not corrupted."
+
+
 class DataHandler:
 
     def __init__(self, dtd_dir):
         self.stack = []
-	self.errors = []
-	self.integers = []
-	self.strings = []
-	self.lists = []
+        self.errors = []
+        self.integers = []
+        self.strings = []
+        self.lists = []
         self.dictionaries = []
         self.structures = {}
         self.items = []
         self.dtd_dir = dtd_dir
-        self.initialized = False
-
-    def run(self, handle):
-        """Set up the parser and let it parse the XML results"""
-        self.parser = expat.ParserCreate()
+        self.valid = True
+        # Set to False once EUtils always returns XML files starting with <!xml
+        self.parser = expat.ParserCreate(namespace_separator=" ")
         self.parser.SetParamEntityParsing(expat.XML_PARAM_ENTITY_PARSING_ALWAYS)
-        self.parser.StartElementHandler = self.startElement
-        self.parser.EndElementHandler = self.endElement
-        self.parser.CharacterDataHandler = self.characters
-        self.parser.ExternalEntityRefHandler = self.external_entity_ref_handler
-        self.parser.ParseFile(handle)
-        self.parser = None
+        self.parser.XmlDeclHandler = self.xmlDeclHandler
+        self.parser.StartElementHandler = self.startElementHandler
+        self.parser.EndElementHandler = self.endElementHandler
+        self.parser.CharacterDataHandler = self.characterDataHandler
+        self.parser.ExternalEntityRefHandler = self.externalEntityRefHandler
+        self.parser.StartNamespaceDeclHandler = self.startNamespaceDeclHandler
+
+    def read(self, handle):
+        """Set up the parser and let it parse the XML results"""
+        try:
+            self.parser.ParseFile(handle)
+        except expat.ExpatError:
+            if self.valid:
+                # We saw the initial <!xml declaration, so we can be sure that
+                # we are parsing XML data. Most likely, the XML file is
+                # corrupted.
+                raise CorruptedXMLError
+            else:
+                # We have not seen the initial <!xml declaration, so probably
+                # the input data is not in XML format.
+                raise NotXMLError
         return self.object
 
-    def startElement(self, name, attrs):
-        if not self.initialized:
-            # This XML file does not have a DTD; load its definitions here
-            # using the first element in the XML. As far as I know, this
-            # occurs only for EFetch results downloaded from the Journals
-            # database.
-            self.load_definitions(name)
+    def parse(self, handle):
+        BLOCK = 1024
+        while True:
+            #Read in another block of the file...
+            text = handle.read(BLOCK)
+            if not text:
+                # We have reached the end of the XML file
+                if self.stack:
+                    raise CorruptedXMLError
+                for record in self.object:
+                    yield record
+                self.parser.Parse("", True)
+                self.parser = None
+                return
+
+            try:
+                self.parser.Parse(text, False)        
+            except expat.ExpatError:
+                if self.valid:
+                    # We saw the initial <!xml declaration, so we can be sure
+                    # that we are parsing XML data. Most likely, the XML file
+                    # is corrupted.
+                    raise CorruptedXMLError
+                else:
+                    # We have not seen the initial <!xml declaration, so
+                    # probably the input data is not in XML format.
+                    raise NotXMLError
+
+            if not self.stack:
+                # Haven't read enough from the XML file yet
+                continue
+
+            records = self.stack[0]
+            if not isinstance(records, list):
+                raise ValueError("The XML file does not represent a list. Please use Entrez.read instead of Entrez.parse")
+            while len(records) > 1: # Then the top record is finished
+                record = records.pop(0)
+                yield record
+
+    def xmlDeclHandler(self, version, encoding, standalone):
+        # The purpose of this method is to make sure that we are parsing XML.
+        self.valid = True
+
+    def startNamespaceDeclHandler(self, prefix, un):
+        raise NotImplementedError("The Bio.Entrez parser cannot handle XML data that make use of XML namespaces")
+
+    def startElementHandler(self, name, attrs):
+        if not self.valid:
+            raise NotXMLError
         self.content = ""
         if name in self.lists:
             object = ListElement()
@@ -110,7 +174,7 @@ class DataHandler:
             object = DictionaryElement()
         elif name in self.structures:
             object = StructureElement(self.structures[name])
-        elif name in self.items:	# Only appears in ESummary
+        elif name in self.items: # Only appears in ESummary
             name = str(attrs["Name"]) # convert from Unicode
             del attrs["Name"]
             itemtype = str(attrs["Type"]) # convert from Unicode
@@ -128,6 +192,9 @@ class DataHandler:
         elif name in self.strings + self.errors + self.integers:
             self.attributes = attrs
             return
+        else:
+            # Element not found in DTD; this will not be stored in the record
+            object = ""
         if object!="":
             object.tag = name
             if attrs:
@@ -140,7 +207,9 @@ class DataHandler:
                     current[name] = object
         self.stack.append(object)
 
-    def endElement(self, name):
+    def endElementHandler(self, name):
+        if not self.valid:
+            raise NotXMLError
         value = self.content
         if name in self.errors:
             if value=="":
@@ -159,7 +228,7 @@ class DataHandler:
             self.object = self.stack.pop()
             if self.object.itemtype in ("List", "Structure"):
                 return
-            elif self.object.itemtype=="Integer":
+            elif self.object.itemtype=="Integer" and value:
                 value = IntegerElement(value)
             else:
                 # Convert Unicode strings to plain strings if possible
@@ -181,7 +250,9 @@ class DataHandler:
         except AttributeError:
             current[name] = value
 
-    def characters(self, content):
+    def characterDataHandler(self, content):
+        if not self.valid:
+            raise NotXMLError
         self.content += content
 
     def elementDecl(self, name, model):
@@ -190,6 +261,8 @@ class DataHandler:
         encountered in a DTD. The purpose of this function is to determine
         whether this element should be regarded as a string, integer, list
         dictionary, structure, or error."""
+        if not self.valid:
+            raise NotXMLError
         if name.upper()=="ERROR":
             self.errors.append(name)
             return
@@ -262,43 +335,48 @@ class DataHandler:
         else:
             self.structures.update({name: multiple})
 
-    def external_entity_ref_handler(self, context, base, systemId, publicId):
+    def externalEntityRefHandler(self, context, base, systemId, publicId):
         """The purpose of this function is to load the DTD locally, instead
         of downloading it from the URL specified in the XML. Using the local
         DTD results in much faster parsing. If the DTD is not found locally,
         we try to download it. In practice, this may fail though, if the XML
         relies on many interrelated DTDs. If new DTDs appear, putting them in
         Bio/Entrez/DTDs will allow the parser to see them."""
-        self.initialized = True
+        if not self.valid:
+            raise NotXMLError
         location, filename = os.path.split(systemId)
         path = os.path.join(self.dtd_dir, filename)
         try:
             handle = open(path)
         except IOError:
-            import warnings, urllib
-            warnings.warn("DTD file %s not found in Biopython installation; trying to retrieve it from NCBI" % filename)
-            handle = urllib.urlopen(systemId)
+            message = """\
+Unable to load DTD file %s.
+
+Bio.Entrez uses NCBI's DTD files to parse XML files returned by NCBI Entrez.
+Though most of NCBI's DTD files are included in the Biopython distribution,
+sometimes you may find that a particular DTD file is missing. In such a
+case, you can download the DTD file from NCBI and install it manually.
+
+Usually, you can find missing DTD files at either
+    http://www.ncbi.nlm.nih.gov/dtd/
+or
+    http://eutils.ncbi.nlm.nih.gov/entrez/query/DTD/
+If you cannot find %s there, you may also try to search
+for it with a search engine such as Google.
+
+Please save %s in the directory
+%s
+in order for Bio.Entrez to find it.
+Alternatively, you can save %s in the directory
+Bio/Entrez/DTDs in the Biopython distribution, and reinstall Biopython.
+
+Please also inform the Biopython developers by sending an email to
+biopython-dev@biopython.org to inform us about this missing DTD, so that we
+can include it with the next release of Biopython.
+""" % (filename, filename, filename, self.dtd_dir, filename)
+            raise RuntimeError(message)
+            
         parser = self.parser.ExternalEntityParserCreate(context)
         parser.ElementDeclHandler = self.elementDecl
         parser.ParseFile(handle)
         return 1
-
-    def load_definitions(self, filename):
-        """This function is only needed if the XML does not specify a DTD.
-        As far as I can tell, this only occurs for EFetch results from the
-        Journals database. Use a hand-written set of definitions instead."""
-        self.initialized = True
-        if filename=="SerialSet":
-            # EFetch results from the Journals database
-            import  SerialSet as module
-        else:
-            import warnings
-            warnings.warn("No parser available for %s; skipping its elements" % filename)
-            return
-        self.errors.extend(module.errors)
-        self.integers.extend(module.integers)
-        self.strings.extend(module.strings)
-        self.lists.extend(module.lists)
-        self.dictionaries.extend(module.dictionaries)
-        self.structures.update(module.structures)
-        self.items.extend(module.items)
