@@ -15,10 +15,8 @@ XXX ref
 
 import numpy
 
-
-# XXX typecodes for Numeric
-# XXX multiprocessor
-
+#These are module level defaults, which in theory users might change
+#at run time to adjust the behaviour of the train function:
 MAX_IIS_ITERATIONS = 10000    # Maximum iterations for IIS.
 IIS_CONVERGE = 1E-5           # Convergence criteria for IIS.
 MAX_NEWTON_ITERATIONS = 100   # Maximum iterations on Newton's method.
@@ -197,7 +195,7 @@ def _iis_solve_delta(N, feature, f_sharp, empirical, prob_yx):
     return delta
 
 def _train_iis(xs, classes, features, f_sharp, alphas, e_empirical):
-    # Do one iteration of hill climbing to find better alphas.
+    """Do one iteration of hill climbing to find better alphas (PRIVATE)."""
     # This is a good function to parallelize.
 
     # Pre-calculate P(y|x)
@@ -212,7 +210,7 @@ def _train_iis(xs, classes, features, f_sharp, alphas, e_empirical):
 
 
 def train(training_set, results, feature_fns, update_fn=None):
-    """train(training_set, results, feature_fns[, update_fn]) -> MaxEntropy object
+    """Train a maximum entropy classifier, returns MaxEntropy object.
 
     Train a maximum entropy classifier on a training set.
     training_set is a list of observations.  results is a list of the
@@ -268,7 +266,7 @@ def train(training_set, results, feature_fns, update_fn=None):
     return me
 
 
-if __name__ == "__main__" :
+if __name__ == "__main__":
     #Car data from example Naive Bayes Classifier example by Eric Meisner November 22, 2003
     #http://www.inf.u-szeged.hu/~ormandi/teaching/mi2/02-naiveBayes-example.pdf
 
@@ -301,7 +299,7 @@ if __name__ == "__main__" :
     #Requires some rules or features
     def udf1(ts, cl):
         if ts[0] =='Red':
-	    return 0
+            return 0
         else:
             return 1
 
@@ -321,5 +319,5 @@ if __name__ == "__main__" :
 
     xe=train(xcar, ycar, user_functions)
     for xv,yv in zip(xcar, ycar):
-	    xc=classify(xe, xv)
-	    print 'Pred:', xv, 'gives', xc, 'y is', yv
+        xc=classify(xe, xv)
+        print 'Pred:', xv, 'gives', xc, 'y is', yv
