@@ -26,7 +26,7 @@ class MMCIF2Dict(UserDict):
         # entry for garbage
         self.data[None]=[]
         if not os.path.isfile(filename):
-            raise IOError, "File not found."
+            raise IOError("File not found.")
         Bio.PDB.mmCIF.MMCIFlex.open_file(filename)
         self._make_mmcif_dict()
         Bio.PDB.mmCIF.MMCIFlex.close_file()
@@ -76,7 +76,8 @@ class MMCIF2Dict(UserDict):
                         token, value=get_token()  
                         # print token, value
                     if pos!=nr_fields-1:
-                        print "ERROR: broken name-data pair (data missing)!"
+                        warnings.warn("ERROR: broken name-data pair "
+                                      "(data missing)!", RuntimeWarning)
                     # The last token was not used, so
                     # don't set token to None! (this means the 
                     # last parsed token goes through the loop again)
@@ -87,7 +88,8 @@ class MMCIF2Dict(UserDict):
                     # print token, value
                     mmcif_dict[value]=data
                     if next_token<4:
-                        print "ERROR: broken name-data pair (name-non data pair)!"  
+                        warnings.warn("ERROR: broken name-data pair "
+                                      "(name-non data pair)!", RuntimeWarning)
                         # print token, value
                     else:   
                         # get next token
@@ -102,8 +104,9 @@ class MMCIF2Dict(UserDict):
                 token=None
             else:
                 # we found some complete garbage
-                print "ERROR: broken name-data pair (missing name)!"
-                print token, value
+                warnings.warn("ERROR: broken name-data pair "
+                              "(missing name)!\n%s %s" % (token, value),
+                              RuntimeWarning)
                 mmcif_dict[None].append(value)
                 # get next token
                 token=None
