@@ -4,14 +4,12 @@
 # as part of this package.  
 
 # Python stuff
-import sys
-from string import split
-
+import warnings
 import numpy
 
 # My stuff
 from StructureBuilder import StructureBuilder
-from PDBExceptions import PDBConstructionException
+from PDBExceptions import PDBConstructionException, PDBConstructionWarning
 from parse_pdb_header import _parse_pdb_header_list
 
 __doc__="Parser for PDB files."
@@ -125,7 +123,7 @@ class PDBParser:
                     model_open=1
                 fullname=line[12:16]
                 # get rid of whitespace in atom names
-                split_list=split(fullname)
+                split_list=fullname.split()
                 if len(split_list)!=1:
                     # atom name has internal spaces, e.g. " N B ", so
                     # we do not strip spaces
@@ -140,7 +138,7 @@ class PDBParser:
                     serial_number=int(line[6:11])
                 except:
                     serial_number=0
-                resseq=int(split(line[22:26])[0])   # sequence identifier   
+                resseq=int(line[22:26].split()[0])   # sequence identifier   
                 icode=line[26:27]           # insertion code
                 if record_type=='HETATM':       # hetero atom flag
                     if resname=="HOH" or resname=="WAT":
@@ -151,14 +149,31 @@ class PDBParser:
                     hetero_flag=" "
                 residue_id=(hetero_flag, resseq, icode)
                 # atomic coordinates
-                x=float(line[30:38]) 
-                y=float(line[38:46]) 
-                z=float(line[46:54])
+                try:
+                    x=float(line[30:38]) 
+                    y=float(line[38:46]) 
+                    z=float(line[46:54])
+                except:
+                    #Should we allow parsing to continue in permissive mode?
+                    #If so what coordindates should we default to?  Easier to abort!
+                    raise PDBContructionError("Invalid or missing coordinate(s) at line %i." \
+                                              % global_line_counter)
                 coord=numpy.array((x, y, z), 'f')
                 # occupancy & B factor
-                occupancy=float(line[54:60])
-                bfactor=float(line[60:66])
+                try:
+                    occupancy=float(line[54:60])
+                except:
+                    self._handle_PDB_exception("Invalid or missing occupancy",
+                                               global_line_counter)
+                    occupancy = 0.0 #Is one or zero a good default?
+                try:
+                    bfactor=float(line[60:66])
+                except:
+                    self._handle_PDB_exception("Invalid or missing B factor",
+                                               global_line_counter)
+                    bfactor = 0.0 #The PDB use a default of zero if the data is missing
                 segid=line[72:76]
+                element=line[76:78].strip()
                 if current_segid!=segid:
                     current_segid=segid
                     structure_builder.init_seg(current_segid)
@@ -180,7 +195,8 @@ class PDBParser:
                         self._handle_PDB_exception(message, global_line_counter) 
                 # init atom
                 try:
-                    structure_builder.init_atom(name, coord, bfactor, occupancy, altloc, fullname, serial_number)
+                    structure_builder.init_atom(name, coord, bfactor, occupancy, altloc,
+                                                fullname, serial_number, element)
                 except PDBConstructionException, message:
                     self._handle_PDB_exception(message, global_line_counter)
             elif(record_type=='ANISOU'):
@@ -226,9 +242,11 @@ class PDBParser:
         """
         message="%s at line %i." % (message, line_counter)
         if self.PERMISSIVE:
-            # just print a warning - some residues/atoms will be missing
-            print "PDBConstructionException: %s" % message
-            print "Exception ignored.\nSome atoms or residues will be missing in the data structure."
+            # just print a warning - some residues/atoms may be missing
+            warnings.warn("PDBConstructionException: %s\n"
+                          "Exception ignored.\n"
+                          "Some atoms or residues may be missing in the data structure."
+                          % message, PDBConstructionWarning)
         else:
             # exceptions are fatal - raise again with new message (including line nr)
             raise PDBConstructionException(message)
@@ -240,7 +258,8 @@ if __name__=="__main__":
 
     p=PDBParser(PERMISSIVE=1)
 
-    s=p.get_structure("scr", sys.argv[1])
+    filename = sys.argv[1]
+    s=p.get_structure("scr", filename)
 
     for m in s:
         p=m.get_parent()
