@@ -250,9 +250,17 @@ class _PPBuilder:
     def _accept(self, residue):
         "Check if the residue is an amino acid."
         if is_aa(residue):
-            # not a standard AA so skip
             return 1
         else:
+            if "CA" in residue.child_dict:
+                #It has an alpha carbon...
+                #We probably need to update the hard coded list of
+                #non-standard residues, see function is_aa for details.
+                import warnings
+                warnings.warn("Assuming residue %s is an unknown modified "
+                              "amino acid" % residue.get_resname())
+                return 1
+            # not a standard AA so skip
             return 0
     
     def build_peptides(self, entity, aa_only=1):
@@ -356,7 +364,7 @@ class PPBuilder(_PPBuilder):
             for cc in clist:
                 # To form a peptide bond, N and C must be 
                 # within radius and have the same altloc
-                # identifier or one altloc blanc
+                # identifier or one altloc blank
                 n_altloc=nn.get_altloc()
                 c_altloc=cc.get_altloc()
                 if n_altloc==c_altloc or n_altloc==" " or c_altloc==" ": 
