@@ -1,9 +1,8 @@
-# Copyright (C) 2002, Thomas Hamelryck (thamelry@vub.ac.be)
+# Copyright (C) 2002, Thomas Hamelryck (thamelry@binf.ku.dk)
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.  
 
-
 # My Stuff
 from Entity import Entity
 
@@ -37,5 +36,22 @@ class Structure(Entity):
         o m1, m2 - Model instances
         """
         return cmp(m1.get_id(), m2.get_id())
+
+    # Public 
+
+    def get_chains(self):
+        for m in self:
+            for c in m:
+                yield c
+
+    def get_residues(self):
+        for c in self.get_chains():
+            for r in c:
+                yield r
+
+    def get_atoms(self):
+        for r in self.get_residues():
+            for a in r:
+                yield a
         
 
