@@ -1,5 +1,7 @@
 #include <Python.h>
 
+char *mmcif_get_string(void);
+
 FILE *fp;
 
 static PyObject *MMCIFlex_open_file(PyObject *self, PyObject *args)
@@ -44,7 +46,7 @@ static PyObject *MMCIFlex_get_token(PyObject *self, PyObject *args)
 	/* if flag==0 we are EOF */
 	if(flag)
 	{
-		value=(char *)mmcif_get_string();
+		value=mmcif_get_string();
 	}	
 
 	/* return the (tokennumber, string) tuple */
