@@ -30,7 +30,6 @@ from copy import copy
 from types import *
 
 from Residues import Residues
-from FileIndex import FileIndex
 
 # This table is taken from the RAF release notes, and includes the
 # undocumented mapping "UNK" -> "X"
@@ -67,22 +66,22 @@ to_one_letter_code= {
     'TPO':'T', 'TPQ':'A', 'TRG':'K', 'TRO':'W', 'TYB':'Y',
     'TYQ':'Y', 'TYS':'Y', 'TYY':'Y', 'AGM':'R', 'GL3':'G',
     'SMC':'C', 'ASX':'B', 'CGU':'E', 'CSX':'C', 'GLX':'Z',
+    'PYX':'C',
     'UNK':'X'
     }
 
 
-def normalize_letters(one_letter_code) :
+def normalize_letters(one_letter_code):
     """Convert RAF one-letter amino acid codes into IUPAC standard codes.
     
     Letters are uppercased, and "." ("Unknown") is converted to "X".
     """
-    if one_letter_code == '.' :
+    if one_letter_code == '.':
         return 'X'
-    else :
+    else:
         return one_letter_code.upper()
 
-
-class SeqMapIndex(FileIndex) :
+class SeqMapIndex(dict):
     """An RAF file index.
 
     The RAF file itself is about 50 MB. This index provides rapid, random
@@ -93,24 +92,49 @@ class SeqMapIndex(FileIndex) :
     chain IDs.    
     """
 
-    def __init__(self, raf_filename) :
-        #We take a shortcut when making the index to avoid having to parse
-        #every line
-        iterator_gen = Iterator
-        key_gen = lambda rec : rec[0:5]
-        FileIndex.__init__(self, raf_filename, iterator_gen, key_gen)
-
-        #Replace Iterator with Parsing version
-        self.iterator_gen = lambda  fh : Iterator(fh, Parser())
+    def __init__(self, filename):
+        """
+        Arguments:
         
+          filename  -- The file to index
+        """
+        dict.__init__(self)
+        self.filename = filename
 
-    def getSeqMap(self, residues) :
+        f = open(self.filename)
+        try:
+            position = 0
+            while True:
+                line = f.readline()
+                if not line: break
+                key = line[0:5]
+                if key != None:
+                    self[key]=position
+                position = f.tell()
+        finally:
+            f.close()
+
+    def __getitem__(self, key):
+        """ Return an item from the indexed file. """
+        position = dict.__getitem__(self,key)
+
+        f = open(self.filename)
+        try:
+            f.seek(position)
+            line = f.readline()
+            record = SeqMap(line)
+        finally:
+            f.close()
+        return record
+
+
+    def getSeqMap(self, residues):
         """Get the sequence map for a collection of residues.
 
         residues -- A Residues instance, or a string that can be converted into
                     a Residues instance.
         """
-        if type(residues) == StringType :
+        if type(residues) == StringType:
             residues = Residues(residues)
 
         pdbid  = residues.pdbid
@@ -118,7 +142,7 @@ class SeqMapIndex(FileIndex) :
         if not frags: frags =(('_','',''),) # All residues of unnamed chain
 
         seqMap = None
-        for frag in frags :
+        for frag in frags:
             chainid = frag[0]
             if chainid=='' or chainid=='-' or chainid==' ' or chainid=='_':
                 chainid = '_'
@@ -135,16 +159,16 @@ class SeqMapIndex(FileIndex) :
             
             sm = sm[start:end]
 
-            if seqMap == None :
+            if seqMap == None:
                 seqMap = sm
-            else :
+            else:
                 seqMap += sm
                             
         return seqMap
 
 
 
-class SeqMap :
+class SeqMap:
     """An ASTRAL RAF (Rapid Access Format) Sequence Map.
     
     This is a list like object; You can find the location of particular residues
@@ -162,29 +186,67 @@ class SeqMap :
     res -- A list of Res objects, one for each residue in this sequence map
     """
 
-    def __init__(self) :
+    def __init__(self, line=None):
         self.pdbid = ''
         self.pdb_datestamp = ''
         self.version = ''
         self.flags = ''
         self.res = []
+        if line:
+            self._process(line)
         
-    def index(self, resid, chainid="_") :
-        for i in range(0, len(self.res)) :
-            if self.res[i].resid == resid and self.res[i].chainid == chainid :
-                return i
-        raise KeyError, "No such residue "+chainid+resid
 
-    def __getslice__(self, i, j) :
+    def _process(self, line):
+        """Parses a RAF record into a SeqMap object.
+        """
+        header_len = 38
+ 
+        line = line.rstrip()  # no trailing whitespace        
+
+        if len(line)<header_len: 
+            raise ValueError("Incomplete header: "+line)
+
+        self.pdbid = line[0:4]
+        chainid = line[4:5]
+        
+        self.version = line[6:10]
+
+        #Raf format versions 0.01 and 0.02 are identical for practical purposes
+        if(self.version != "0.01" and  self.version !="0.02"):
+            raise ValueError("Incompatible RAF version: "+self.version)
+
+        self.pdb_datestamp = line[14:20]
+        self.flags = line[21:27]
+
+        for i in range(header_len, len(line), 7):
+            f = line[i : i+7]
+            if len(f)!=7:
+                raise ValueError("Corrupt Field: ("+f+")")
+            r = Res()
+            r.chainid = chainid
+            r.resid =  f[0:5].strip()
+            r.atom = normalize_letters(f[5:6])
+            r.seqres = normalize_letters(f[6:7])
+
+            self.res.append(r)
+
+
+    def index(self, resid, chainid="_"):
+        for i in range(0, len(self.res)):
+            if self.res[i].resid == resid and self.res[i].chainid == chainid:
+                return i
+        raise KeyError("No such residue "+chainid+resid)
+
+    def __getslice__(self, i, j):
         s = copy(self)
         s.res = s.res[i:j]
         return s
 
-    def append(self, res) :
+    def append(self, res):
         """Append another Res object onto the list of residue mappings."""
         self.res.append(res)
 
-    def extend(self, other) :
+    def extend(self, other):
         """Append another SeqMap onto the end of self.
 
         Both SeqMaps must have the same PDB ID, PDB datestamp and
@@ -192,27 +254,27 @@ class SeqMap :
         may happen when fragments are taken from different chains.
         """
         if not isinstance(other, SeqMap):
-            raise TypeError, "Can only extend a SeqMap with a SeqMap."
-        if self.pdbid != other.pdbid :
-            raise TypeError, "Cannot add fragments from different proteins"
-        if self.version != other.version :
-            raise TypeError, "Incompatible rafs"
-        if self.pdb_datestamp != other.pdb_datestamp :
-            raise TypeError, "Different pdb dates!"
-        if self.flags != other.flags :
+            raise TypeError("Can only extend a SeqMap with a SeqMap.")
+        if self.pdbid != other.pdbid:
+            raise TypeError("Cannot add fragments from different proteins")
+        if self.version != other.version:
+            raise TypeError("Incompatible rafs")
+        if self.pdb_datestamp != other.pdb_datestamp:
+            raise TypeError("Different pdb dates!")
+        if self.flags != other.flags:
             self.flags = ''
         self.res += other.res
 
-    def __iadd__(self, other) :
+    def __iadd__(self, other):
         self.extend(other)
         return self
 
-    def __add__(self, other) :
+    def __add__(self, other):
         s = copy(self)
         s.extend(other)
         return s
 
-    def getAtoms(self, pdb_handle, out_handle) :
+    def getAtoms(self, pdb_handle, out_handle):
         """Extract all relevant ATOM and HETATOM records from a PDB file.
 
         The PDB file is scanned for ATOM and HETATOM records. If the
@@ -231,7 +293,7 @@ class SeqMap :
 
         #The set of residues that I have to find records for. 
         resSet = {}
-        for r in self.res :
+        for r in self.res:
             if r.atom=='X' : #Unknown residue type
                 continue
             chainid = r.chainid
@@ -241,31 +303,31 @@ class SeqMap :
             resSet[(chainid,resid)] = r
 
         resFound = {}
-        for line in pdb_handle.xreadlines() :
-            if line.startswith("ATOM  ") or line.startswith("HETATM") :
+        for line in pdb_handle.xreadlines():
+            if line.startswith("ATOM  ") or line.startswith("HETATM"):
                 chainid = line[21:22]
                 resid = line[22:27].strip()
                 key = (chainid, resid)
-                if resSet.has_key(key):
+                if key in resSet:
                     res = resSet[key]
                     atom_aa = res.atom
                     resName = line[17:20]
-                    if to_one_letter_code.has_key(resName) :
-                        if to_one_letter_code[resName] == atom_aa :
+                    if resName in to_one_letter_code:
+                        if to_one_letter_code[resName] == atom_aa:
                             out_handle.write(line)
                             resFound[key] = res
 
-        if len(resSet) != len(resFound) :
+        if len(resSet) != len(resFound):
             #for k in resFound.keys():
             #    del resSet[k]
             #print resSet
                                      
-            raise RuntimeError, 'I could not find at least one ATOM or ' \
-               +'HETATM record for each and every residue in this sequence map.'
+            raise RuntimeError('I could not find at least one ATOM or HETATM' \
+                   +' record for each and every residue in this sequence map.')
         
         
         
-class Res :
+class Res:
     """ A single residue mapping from a RAF record.
 
     chainid -- A single character chain ID.
@@ -276,91 +338,20 @@ class Res :
 
     seqres  -- amino acid one-letter code from SEQRES records.
     """
-    def __init__(self) :
+    def __init__(self):
         self.chainid = ''
         self.resid = ''
         self.atom = ''
         self.seqres = ''
 
 
+def parse(handle):
+    """Iterates over a RAF file, returning a SeqMap object for each line
+    in the file.
 
-class Iterator:
-    """Iterates over a RAF file.
-    """
-    def __init__(self, handle, parser=None):
-        """Create an object that iterates over a RAF file.
-
+    Arguments:
+        
         handle -- file-like object.
-        
-        parser -- an optional Parser object to change the results into
-                  another form.  If set to None, then the raw contents
-                  of the file will be returned.
-                  
-        """
-        if type(handle) is not FileType and type(handle) is not InstanceType:
-            raise TypeError, "I expected a file handle or file-like object"
-        self._handle = handle
-        self._parser = parser
-
-    def next(self):
-        """Retrieve the next record."""
-        while 1:
-            line = self._handle.readline()
-            if not line: return None
-            if line[0] !='#':  break  # Not a comment line
-        if self._parser is not None :    
-            return self._parser.parse(line)
-        return line
-    
-    def __iter__(self):
-        return iter(self.next, None)
-
-
-class Parser:
-    """Parses a RAF record into a SeqMap object.
-    """
-    def parse(self, line):
-        """Returns a SeqMap """
-        header_len = 38
- 
-        seqMap = SeqMap()
-        line = line.rstrip()  # no trailing whitespace        
-
-        if len(line)<header_len: 
-            raise ValueError, "Incomplete header: "+rafLine
-
-        seqMap.pdbid = line[0:4]
-        chainid = line[4:5]
-        
-        seqMap.version = line[6:10]
-
-        #Raf format versions 0.01 and 0.02 are identical for practical purposes
-        if(seqMap.version != "0.01" and  seqMap.version !="0.02") :
-            raise ValueError, "Incompatible RAF version: "+seqMap.version 
-
-        seqMap.pdb_datestamp = line[14:20]
-        seqMap.flags = line[21:27]
-
-        for i in range(header_len, len(line), 7) :
-            f = line[i : i+7]
-            if len(f)!=7:
-                raise ValueError, "Corrupt Field: ("+f+")"
-            r = Res()
-            r.chainid = chainid
-            r.resid =  f[0:5].strip()
-            r.atom = normalize_letters(f[5:6])
-            r.seqres = normalize_letters(f[6:7])
-
-            seqMap.res.append(r)
-
-        return seqMap
-
-
-
-
-
-
-
-
-
-
+    """ 
+    for line in handle:
+        yield SeqMap(line)
