@@ -33,16 +33,19 @@ o FeatureLocation - Specify the start and end location of a feature.
 
 o ExactPosition - Specify the position as being exact.
 o WithinPosition - Specify a position occuring within some range.
-o BetweenPosition - Specify a position occuring between a range.
+o BetweenPosition - Specify a position occuring between a range (OBSOLETE?).
 o BeforePosition - Specify the position as being found before some base.
 o AfterPosition - Specify the position as being found after some base.
+o OneOfPosition - Specify a position where the location can be multiple positions.
 """
 
-class SeqFeature:
+from Bio.Seq import MutableSeq, reverse_complement
+
+class SeqFeature(object):
     """Represent a Sequence Feature on an object.
 
     Attributes:
-    o location - the location of the feature on the sequence
+    o location - the location of the feature on the sequence (FeatureLocation)
     o type - the specified type of the feature (ie. CDS, exon, repeat...)
     o location_operator - a string specifying how this SeqFeature may
     be related to others. For example, in the example GenBank feature
@@ -55,7 +58,7 @@ class SeqFeature:
     o ref - A reference to another sequence. This could be an accession
     number for some different sequence.
     o ref_db - A different database for the reference accession number.
-    o qualifier - A dictionary of qualifiers on the feature. These are
+    o qualifiers - A dictionary of qualifiers on the feature. These are
     analagous to the qualifiers from a GenBank feature table. The keys of
     the dictionary are qualifier names, the values are the qualifier
     values.
@@ -67,42 +70,74 @@ class SeqFeature:
     The the top level feature would be a CDS from 1 to 60, and the sub
     features would be of 'CDS_join' type and would be from 1 to 10, 30 to
     40 and 50 to 60, respectively.
+
+    To get the nucleotide sequence for this CDS, you would need to take the
+    parent sequence and do seq[0:10]+seq[29:40]+seq[49:60] (Python counting).
+    Things are more complicated with strands and fuzzy positions. To save you
+    dealing with all these special cases, the SeqFeature provides an extract
+    method to do this for you.
     """
     def __init__(self, location = None, type = '', location_operator = '',
                  strand = None, id = "<unknown id>", 
-                 qualifiers = {}, sub_features = [],
+                 qualifiers = None, sub_features = None,
                  ref = None, ref_db = None):
         """Initialize a SeqFeature on a Sequence.
+
+        location can either be a FeatureLocation (with strand argument also
+        given if required), or a Python slice (with strand given as the step).
+
+        e.g. With no strand, on the forward strand, and on the reverse strand:
+
+        >>> from Bio.SeqFeature import SeqFeature, FeatureLocation
+        >>> f1 = SeqFeature(FeatureLocation(5,10), type="domain")
+        >>> f2 = SeqFeature(FeatureLocation(7,110), strand=1, type="CDS")
+        >>> f3 = SeqFeature(FeatureLocation(9,108), strand=-1, type="CDS")
+
+        An invalid strand will trigger an exception:
+
+        >>> f4 = SeqFeature(FeatureLocation(50,60), strand=2)
+        Traceback (most recent call last):
+           ...
+        ValueError: Strand should be +1, -1, 0 or None, not 2
+
+        For exact start/end positions, an integer can be used (as shown above)
+        as shorthand for the ExactPosition object. For non-exact locations, the
+        FeatureLocation must be specified via the appropriate position objects.
         """
+        if strand not in [-1, 0, 1, None] :
+            raise ValueError("Strand should be +1, -1, 0 or None, not %s" \
+                             % repr(strand))
+        if location and not isinstance(location, FeatureLocation):
+            raise TypeError("FeatureLocation (or None) required for the location")
         self.location = location
 
         self.type = type
         self.location_operator = location_operator
         self.strand = strand
         self.id = id
-        # XXX right now sub_features and qualifiers cannot be set 
-        # from the initializer because this causes all kinds 
-        # of recursive import problems. I can't understand why this is
-        # at all :-<
-        self.qualifiers = {}
-        self.sub_features = []
+        if qualifiers is None:
+            qualifiers = {}
+        self.qualifiers = qualifiers
+        if sub_features is None:
+            sub_features = []
+        self.sub_features = sub_features
         self.ref = ref 
         self.ref_db = ref_db
 
     def __repr__(self):
         """A string representation of the record for debugging."""
-        answer = "%s(%s" % (self.__class__, repr(self.location))
-        if self.type :
+        answer = "%s(%s" % (self.__class__.__name__, repr(self.location))
+        if self.type:
             answer += ", type=%s" % repr(self.type)
-        if self.location_operator :
+        if self.location_operator:
             answer += ", location_operator=%s" % repr(self.location_operator)
-        if self.strand :
+        if self.strand:
             answer += ", strand=%s" % repr(self.strand)
-        if self.id and self.id != "<unknown id>" :
+        if self.id and self.id != "<unknown id>":
             answer += ", id=%s" % repr(self.id)
-        if self.ref :
+        if self.ref:
             answer += ", ref=%s" % repr(self.ref)
-        if self.ref_db :
+        if self.ref_db:
             answer += ", ref_db=%s" % repr(self.ref_db)
         answer += ")"
         return answer
@@ -118,7 +153,7 @@ class SeqFeature:
         qualifier_keys = self.qualifiers.keys()
         qualifier_keys.sort()
         for qual_key in qualifier_keys:
-            out += "\tKey: %s, Value: %s\n" % (qual_key,
+            out += "    Key: %s, Value: %s\n" % (qual_key,
                                                self.qualifiers[qual_key])
         if len(self.sub_features) != 0:
             out += "Sub-Features\n"
@@ -127,10 +162,85 @@ class SeqFeature:
 
         return out
 
+    def _shift(self, offset):
+        """Returns a copy of the feature with its location shifted (PRIVATE).
+
+        The annotation qaulifiers are copied."""
+        answer = SeqFeature(location = self.location._shift(offset),
+                   type = self.type,
+                   location_operator = self.location_operator,
+                   strand = self.strand,
+                   id = self.id,
+                   #qualifiers = dict(self.qualifiers.iteritems()),
+                   #sub_features = [f._shift(offset) for f in self.sub_features],
+                   ref = self.ref,
+                   ref_db = self.ref_db)
+        #TODO - Sort out the use of sub_feature and qualifiers in __init___
+        answer.sub_features = [f._shift(offset) for f in self.sub_features]
+        answer.qualifiers = dict(self.qualifiers.iteritems())
+        return answer
+
+    def extract(self, parent_sequence):
+        """Extract feature sequence from the supplied parent sequence.
+
+        The parent_sequence can be a Seq like object or a string, and will
+        generally return an object of the same type. The exception to this is
+        a MutableSeq as the parent sequence will return a Seq object.
+
+        This should cope with complex locations including complements, joins
+        and fuzzy positions. Even mixed strand features should work! This
+        also covers features on protein sequences (e.g. domains), although
+        here reverse strand features are not permitted.
+
+        >>> from Bio.Seq import Seq
+        >>> from Bio.Alphabet import generic_protein
+        >>> from Bio.SeqFeature import SeqFeature, FeatureLocation
+        >>> seq = Seq("MKQHKAMIVALIVICITAVVAAL", generic_protein)
+        >>> f = SeqFeature(FeatureLocation(8,15), type="domain")
+        >>> f.extract(seq)
+        Seq('VALIVIC', ProteinAlphabet())
+
+        Note - currently only sub-features of type "join" are supported.
+        """
+        if isinstance(parent_sequence, MutableSeq):
+            #This avoids complications with reverse complements
+            #(the MutableSeq reverse complement acts in situ)
+            parent_sequence = parent_sequence.toseq()
+        if self.sub_features:
+            if self.location_operator!="join":
+                raise ValueError(f.location_operator)
+            if self.strand == -1:
+                #This is a special case given how the GenBank parser works.
+                #Must avoid doing the reverse complement twice.
+                parts = []
+                for f_sub in self.sub_features:
+                    assert f_sub.strand==-1
+                    parts.append(parent_sequence[f_sub.location.nofuzzy_start:\
+                                                 f_sub.location.nofuzzy_end])
+            else:
+                #This copes with mixed strand features:
+                parts = [f_sub.extract(parent_sequence) \
+                         for f_sub in self.sub_features]
+            #We use addition rather than a join to avoid alphabet issues:
+            f_seq = parts[0]
+            for part in parts[1:] : f_seq += part
+        else:
+            f_seq = parent_sequence[self.location.nofuzzy_start:\
+                                    self.location.nofuzzy_end]
+        if self.strand == -1:
+            #TODO - MutableSeq?
+            try:
+                f_seq = f_seq.reverse_complement()
+            except AttributeError:
+                assert isinstance(f_seq, str)
+                f_seq = reverse_complement(f_seq)
+        return f_seq
+        
+
 # --- References
 
 # TODO -- Will this hold PubMed and Medline information decently?
-class Reference:
+class Reference(object):
     """Represent a Generic Reference object.
 
     Attributes:
@@ -169,12 +279,16 @@ class Reference:
         out += "medline id: %s\n" % self.medline_id
         out += "pubmed id: %s\n" % self.pubmed_id
         out += "comment: %s\n" % self.comment
-
         return out
 
+    def __repr__(self):
+        #TODO - Update this is __init__ later accpets values
+        return "%s(title=%s, ...)" % (self.__class__.__name__,
+                                      repr(self.title))
+
 # --- Handling feature locations
 
-class FeatureLocation:
+class FeatureLocation(object):
     """Specify the location of a feature along a sequence.
 
     This attempts to deal with fuzziness of position ends, but also
@@ -199,6 +313,23 @@ class FeatureLocation:
         position. In the case of integers, the values are assumed to be
         exact and are converted in ExactPosition arguments. This is meant
         to make it easy to deal with non-fuzzy ends.
+
+        i.e. Short form:
+        
+        >>> from Bio.SeqFeature import FeatureLocation
+        >>> loc = FeatureLocation(5,10)
+        
+        Explicit form:
+
+        >>> from Bio.SeqFeature import FeatureLocation, ExactPosition
+        >>> loc = FeatureLocation(ExactPosition(5),ExactPosition(10))
+
+        Other fuzzy positions are used similarly,
+
+        >>> from Bio.SeqFeature import FeatureLocation
+        >>> from Bio.SeqFeature import BeforePosition, AfterPosition
+        >>> loc2 = FeatureLocation(BeforePosition(5),AfterPosition(10))
+
         """
         if isinstance(start, AbstractPosition):
             self._start = start
@@ -222,59 +353,64 @@ class FeatureLocation:
     def __repr__(self):
         """A string representation of the location for debugging."""
         return "%s(%s,%s)" \
-               % (self.__class__, repr(self.start), repr(self.end))
+               % (self.__class__.__name__, repr(self.start), repr(self.end))
 
-    def __getattr__(self, attr):
-        """Make it easy to get non-fuzzy starts and ends.
+    def _shift(self, offset):
+        """Returns a copy of the location shifted by the offset (PRIVATE)."""
+        return FeatureLocation(start = self._start._shift(offset),
+                               end = self._end._shift(offset))
 
-        We override get_attribute here so that in non-fuzzy cases we
-        can just return the start and end position without any hassle.
+    start = property(fget= lambda self : self._start,
+                 doc="Start location (possibly a fuzzy position, read only).")
 
-        To get fuzzy start and ends, just ask for item.start and
-        item.end. To get non-fuzzy attributes (ie. the position only)
-        ask for 'item.nofuzzy_start', 'item.nofuzzy_end'. These should return
+    end = property(fget= lambda self : self._end,
+                   doc="End location (possibly a fuzzy position, read only).")
+
+    def _get_nofuzzy_start(self):
+        #TODO - Do we still use the BetweenPosition class?
+        if ((self._start == self._end) and isinstance(self._start,
+             BetweenPosition)):
+            return self._start.position
+        else:
+            return min(self._start.position,
+                       self._start.position + self._start.extension)
+    nofuzzy_start = property(fget=_get_nofuzzy_start,
+        doc="""Start position (integer, approximated if fuzzy, read only).
+
+        To get non-fuzzy attributes (ie. the position only) ask for
+        'location.nofuzzy_start', 'location.nofuzzy_end'. These should return
         the largest range of the fuzzy position. So something like:
         (10.20)..(30.40) should return 10 for start, and 40 for end.
+        """)
 
-        The special tricky case where is when we have a single between position
-        argument like 2^3 for the range. We want nofuzzy_start and nofuzzy_end
-        to give a reasonable approximation of what this really means, which
-        is an empty string -- so the same position for both. Doing a special
-        case here sucks, but there is really not a general rule you can apply
-        to this.
-        """
-        if attr == 'start':
-            return self._start
-        elif attr == 'end':
-            return self._end
-        elif attr == 'nofuzzy_start':
-            if ((self._start == self._end) and isinstance(self._start,
-                 BetweenPosition)):
-                return self._start.position
-            else:
-                return min(self._start.position,
-                           self._start.position + self._start.extension)
-        elif attr == 'nofuzzy_end':
-            if ((self._start == self._end) and isinstance(self._start,
-                 BetweenPosition)):
-                return self._end.position
-            else:
-                return max(self._end.position,
-                           self._end.position + self._end.extension)
+    def _get_nofuzzy_end(self):
+        #TODO - Do we still use the BetweenPosition class?
+        if ((self._start == self._end) and isinstance(self._start,
+             BetweenPosition)):
+            return self._end.position
         else:
-            raise AttributeError("Cannot evaluate attribute %s." % attr)
+            return max(self._end.position,
+                       self._end.position + self._end.extension)
+    nofuzzy_end = property(fget=_get_nofuzzy_end,
+        doc="""End position (integer, approximated if fuzzy, read only).
 
-class AbstractPosition:
+        To get non-fuzzy attributes (ie. the position only) ask for
+        'location.nofuzzy_start', 'location.nofuzzy_end'. These should return
+        the largest range of the fuzzy position. So something like:
+        (10.20)..(30.40) should return 10 for start, and 40 for end.
+        """)
+
+class AbstractPosition(object):
     """Abstract base class representing a position.
     """
     def __init__(self, position, extension):
         self.position = position
         self.extension = extension
 
-    def __repr__(self) :
+    def __repr__(self):
         """String representation of the location for debugging."""
-        return "%s(%s,%s)" \
-               % (self.__class__, repr(self.position), repr(self.extension))
+        return "%s(%s,%s)" % (self.__class__.__name__, \
+                              repr(self.position), repr(self.extension))
 
     def __cmp__(self, other):
         """A simple comparison function for positions.
@@ -286,7 +422,11 @@ class AbstractPosition:
         assert isinstance(other, AbstractPosition), \
           "We can only do comparisons between Biopython Position objects."
 
-        return cmp(self.position, other.position)  
+        return cmp(self.position, other.position)
+
+    def _shift(self, offset):
+        #We want this to maintain the subclass when called from a subclass
+        return self.__class__(self.position + offset, self.extension)
             
 class ExactPosition(AbstractPosition):
     """Specify the specific position of a boundary.
@@ -304,10 +444,10 @@ class ExactPosition(AbstractPosition):
                                  % extension)
         AbstractPosition.__init__(self, position, 0)
 
-    def __repr__(self) :
+    def __repr__(self):
         """String representation of the ExactPosition location for debugging."""
         assert self.extension == 0
-        return "%s(%s)" % (self.__class__, repr(self.position))
+        return "%s(%s)" % (self.__class__.__name__, repr(self.position))
 
     def __str__(self):
         return str(self.position)
@@ -331,7 +471,7 @@ class WithinPosition(AbstractPosition):
         return "(%s.%s)" % (self.position, self.position + self.extension)
 
 class BetweenPosition(AbstractPosition):
-    """Specify the position of a boundary between two coordinates.
+    """Specify the position of a boundary between two coordinates (OBSOLETE?).
 
     Arguments:
     o position - The start position of the boundary.
@@ -366,10 +506,10 @@ class BeforePosition(AbstractPosition):
                                  % extension)
         AbstractPosition.__init__(self, position, 0)
 
-    def __repr__(self) :
+    def __repr__(self):
         """A string representation of the location for debugging."""
         assert self.extension == 0
-        return "%s(%s)" % (self.__class__, repr(self.position))
+        return "%s(%s)" % (self.__class__.__name__, repr(self.position))
 
     def __str__(self):
         return "<%s" % self.position
@@ -392,10 +532,10 @@ class AfterPosition(AbstractPosition):
                                  % extension)
         AbstractPosition.__init__(self, position, 0)
 
-    def __repr__(self) :
+    def __repr__(self):
         """A string representation of the location for debugging."""
         assert self.extension == 0
-        return "%s(%s)" % (self.__class__, repr(self.position))
+        return "%s(%s)" % (self.__class__.__name__, repr(self.position))
 
     def __str__(self):
         return ">%s" % self.position
@@ -432,9 +572,10 @@ class OneOfPosition(AbstractPosition):
         # initialize with our definition of position and extension
         AbstractPosition.__init__(self, smallest, largest - smallest)
 
-    def __repr__(self) :
+    def __repr__(self):
         """String representation of the OneOfPosition location for debugging."""
-        return "%s(%s)" % (self.__class__, repr(self.position_choices))
+        return "%s(%s)" % (self.__class__.__name__, \
+                           repr(self.position_choices))
 
     def __str__(self):
         out = "one-of("
@@ -443,8 +584,12 @@ class OneOfPosition(AbstractPosition):
         # replace the last comma with the closing parenthesis
         out = out[:-1] + ")"
         return out
-              
-class PositionGap:
+
+    def _shift(self, offset):
+        return self.__class__([position_choice._shift(offset) \
+                               for position_choice in self.position_choices])
+
+class PositionGap(object):
     """Simple class to hold information about a gap between positions.
     """
     def __init__(self, gap_size):
@@ -452,10 +597,20 @@ class PositionGap:
         """
         self.gap_size = gap_size
 
-    def __repr__(self) :
+    def __repr__(self):
         """A string representation of the position gap for debugging."""
-        return "%s(%s)" % (self.__class__, repr(self.gap_size))
+        return "%s(%s)" % (self.__class__.__name__, repr(self.gap_size))
     
     def __str__(self):
         out = "gap(%s)" % self.gap_size
         return out
+
+def _test():
+    """Run the Bio.SeqFeature module's doctests."""
+    print "Runing doctests..."
+    import doctest
+    doctest.testmod()
+    print "Done"
+
+if __name__ == "__main__":
+    _test()
