@@ -11,7 +11,6 @@
 
 import re, time
 from Bio import SeqIO
-from Bio import Translate
 from Bio.Seq import Seq
 from Bio import Alphabet
 from Bio.Alphabet import IUPAC
@@ -25,8 +24,13 @@ from Bio.Data import IUPACData, CodonTable
 
 def reverse(seq):
     """Reverse the sequence. Works on string sequences.
+
+    e.g.
+    >>> reverse("ACGGT")
+    'TGGCA'
+    
     """
-    r = map(None, seq)
+    r = list(seq)
     r.reverse()
     return ''.join(r)
 
@@ -40,19 +44,15 @@ def GC(seq):
     >>> from Bio.SeqUtils import GC
     >>> GC("ACTGN")
     40.0
+
+    Note that this will return zero for an empty sequence.
     """
-    #      19/8/03: Iddo: added provision for lowercase
-    #      19/8/03: Iddo: divide by the sequence's length rather than by the
-    #      A+T+G+C number. In that way, make provision for N.
+    try:
+        gc = sum(map(seq.count,['G','C','g','c','S','s']))
+        return gc*100.0/len(seq)
+    except ZeroDivisionError:
+        return 0.0
         
-    d = {}
-    for nt in ['A','T','G','C','a','t','g','c','S','s']:
-        d[nt] = seq.count(nt)
-        gc = d.get('G',0) + d.get('C',0) + d.get('g',0) + d.get('c',0) + \
-             d.get('S',0) + d.get('s',0)
- 
-    if gc == 0: return 0
-    return gc*100.0/len(seq)
     
 def GC123(seq):
     """Calculates total G+C content plus first, second and third positions.
@@ -112,21 +112,12 @@ def GC_skew(seq, window = 100):
         values.append(skew)
     return values
 
-# 8/19/03 Iddo: moved these imports from within the function as
-# ``import * '' is only
-# allowed within the module
-# Brad -- added an import check so you don't have to have Tkinter
-# installed to use other sequtils functions. A little ugly but
-# it really should be fixed by not using 'import *' which I'm not
-# really excited about doing right now.
-try:
-    from Tkinter import *
-except ImportError:
-    pass
 from math import pi, sin, cos, log
 def xGC_skew(seq, window = 1000, zoom = 100,
                          r = 300, px = 100, py = 100):
     """Calculates and plots normal and accumulated GC skew (GRAPHICS !!!)."""
+    from Tkinter import Scrollbar, Canvas, BOTTOM, BOTH, ALL, \
+                        VERTICAL, HORIZONTAL, RIGHT, LEFT, X, Y
     yscroll = Scrollbar(orient = VERTICAL)
     xscroll = Scrollbar(orient = HORIZONTAL)
     canvas = Canvas(yscrollcommand = yscroll.set,
@@ -186,12 +177,7 @@ def molecular_weight(seq):
     """Calculate the molecular weight of a DNA sequence."""
     if type(seq) == type(''): seq = Seq(seq, IUPAC.unambiguous_dna)
     weight_table = IUPACData.unambiguous_dna_weights
-    #TODO, use a generator expession once we drop Python 2.3?
-    #e.g. return sum(weight_table[x] for x in seq)
-    total = 0
-    for x in seq:
-        total += weight_table[x]
-    return total
+    return sum(weight_table[x] for x in seq)
 
 def nt_search(seq, subseq):
     """Search for a DNA subseq in sequence.
@@ -291,44 +277,6 @@ def seq3(seq):
 ######################
 # {{{ 
 
-def translate(seq, frame = 1, genetic_code = 1, translator = None):
-    """Translation of DNA in one of the six different reading frames (DEPRECATED).
-
-    Use the Bio.Seq.Translate function, or the Seq object's translate method
-    instead:
-
-    >>> from Bio.Seq import Seq
-    >>> my_seq = Seq("AUGGCCAUUGUAAUGGGCCGCUGAAAGGGUGCCCGAUAG")
-    >>> my_seq = Seq("AUGGCCAUUGUAAUGGGCCGCUGAAAGGGUGCCCGAUAGUA")
-    >>> for frame in [0,1,2] :
-    ...    print my_seq[frame:].translate()
-    ... 
-    MAIVMGR*KGAR*
-    WPL*WAAERVPDS
-    GHCNGPLKGCPIV
-    >>> for frame in [0,1,2] :
-    ...     print my_seq.reverse_complement()[frame:].translate()
-    ... 
-    YYRAPFQRPITMA
-    TIGHPFSGPLQWP
-    LSGTLSAAHYNGH
-    """
-    import warnings
-    warnings.warn("Bio.SeqUtils.translate() has been deprecated, and we intend" \
-                  +" to remove it in a future release of Biopython.  Please use"\
-                  +" the method or function in Bio.Seq instead, as described in"\
-                  +" the Tutorial.", DeprecationWarning)
-
-    if frame not in [1,2,3,-1,-2,-3]:
-        raise ValueError('invalid frame')
-
-    if not translator:
-        table = makeTableX(CodonTable.ambiguous_dna_by_id[genetic_code])
-        translator = Translate.Translator(table)
-
-    #Does this frame calculation do something sensible?  No RC taken!
-    return translator.translate(Seq(seq[frame-1:], IUPAC.ambiguous_dna)).data
-
 def GC_Frame(seq, genetic_code = 1):
     """Just an alias for six_frame_translations (OBSOLETE).
 
@@ -343,8 +291,8 @@ def six_frame_translations(seq, genetic_code = 1):
     similar to DNA Striders six-frame translation
 
     e.g.
-    >>> from Bio.SeqUtils import six_frame_translations
-    >>> print six_frame_translations("AUGGCCAUUGUAAUGGGCCGCUGA")
+    from Bio.SeqUtils import six_frame_translations
+    print six_frame_translations("AUGGCCAUUGUAAUGGGCCGCUGA")
     """
     from Bio.Seq import reverse_complement, translate
     anti = reverse_complement(seq)
