@@ -1,5 +1,5 @@
 # Copyright 2002 by Andrew Dalke.  All rights reserved.
-# Revisions 2007-2008 by Peter Cock.
+# Revisions 2007-2009 copyright by Peter Cock.  All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
@@ -33,14 +33,30 @@ class Generic_dbutils:
         cursor.execute(sql)
         rv = cursor.fetchone()
         return rv[0]
+    
+    def execute(self, cursor, sql, args=None):
+        """Just execute an sql command.
+        """
+        cursor.execute(sql, args or ())
 
     def autocommit(self, conn, y = 1):
         # Let's hope it was not really needed
         pass
 
+class Sqlite_dbutils(Generic_dbutils):
+    """Custom database utilities for SQLite.
+    """
+    def execute(self, cursor, sql, args=None):
+        """Replace %s with ? for variable substitution in sqlite3.
+        """
+        sql = sql.replace("%s", "?")
+        cursor.execute(sql, args or ())
+
+_dbutils["sqlite3"] = Sqlite_dbutils
+
 class Mysql_dbutils(Generic_dbutils):
     def last_id(self, cursor, table):
-        try :
+        try:
             #This worked on older versions of MySQL
             return cursor.insert_id()
         except AttributeError:
@@ -68,7 +84,17 @@ class Psycopg_dbutils(Generic_dbutils):
 
     def autocommit(self, conn, y = True):
         conn.autocommit(y)
+
 _dbutils["psycopg"] = Psycopg_dbutils
+ 
+class Psycopg2_dbutils(Psycopg_dbutils):
+    def autocommit(self, conn, y = True):
+        if y:
+            conn.set_isolation_level(0)
+        else:
+            conn.set_isolation_level(1)
+
+_dbutils["psycopg2"] = Psycopg2_dbutils
 
 class Pgdb_dbutils(Generic_dbutils):
     """Add support for pgdb in the PyGreSQL database connectivity package.
