@@ -1,28 +1,27 @@
-#TODO - What if numpy was installed after Biopython,
-#meaning Cluster's C code hasn't been compiled?
-try :
-    import numpy
-except ImportError :
-    from Bio import MissingExternalDependencyError
-    raise MissingExternalDependencyError(\
-        "Install NumPy if you want to use Bio.Cluster.")
+# This code is part of the Biopython distribution and governed by its
+# license.  Please see the LICENSE file that should have been included
+# as part of this package.
 
 import unittest
-import sys
 
-def run_tests(module="Pycluster"):
-    if module==[]:
-        module = "Bio.Cluster"
-    if not module in ("Pycluster", "Bio.Cluster"):
-        raise ValueError('Unknown module name: ' + module)
-    TestCluster.module = module
-    suite = unittest.TestLoader().loadTestsFromTestCase(TestCluster)
-    runner = unittest.TextTestRunner(sys.stdout, verbosity = 2)
-    runner.run(suite)
+try:
+    from Bio import Cluster
+except ImportError:
+    from Bio import MissingExternalDependencyError
+    raise MissingExternalDependencyError(\
+        "If you want to use Bio.Cluster, install NumPy first and then reinstall Biopython")
 
+try:
+    import numpy
+except ImportError:
+    from Bio import MissingExternalDependencyError
+    raise MissingExternalDependencyError(\
+        "Install NumPy if you want to use Bio.Cluster")
 
 class TestCluster(unittest.TestCase):
 
+    module = 'Bio.Cluster'
+
     def test_median_mean(self):
         if TestCluster.module=='Bio.Cluster':
             from Bio.Cluster import mean, median
@@ -131,7 +130,10 @@ class TestCluster(unittest.TestCase):
                              [ 1, 1, 1, 1, 1], 
                              [ 1, 1, 1, 1, 1]], int) 
       
-        clusterid, error, nfound = kcluster(data, nclusters=nclusters, mask=mask, weight=weight, transpose=0, npass=100, method='a', dist='e')
+        clusterid, error, nfound = kcluster(data, nclusters=nclusters,
+                                            mask=mask, weight=weight,
+                                            transpose=0, npass=100,
+                                            method='a', dist='e')
         self.assertEqual(len(clusterid), len(data))
 
         correct = [0,1,1,2]
@@ -168,7 +170,9 @@ class TestCluster(unittest.TestCase):
                             [ 1, 1 ],
                             [ 1, 1 ]], int)
 
-        clusterid, error, nfound = kcluster(data, nclusters=3, mask=mask, weight=weight, transpose=0, npass=100, method='a', dist='e')
+        clusterid, error, nfound = kcluster(data, nclusters=3, mask=mask,
+                                            weight=weight, transpose=0,
+                                            npass=100, method='a', dist='e')
         self.assertEqual(len(clusterid), len(data))
 
         correct = [0, 0, 0, 0, 0, 0, 1, 1, 2, 1, 1, 1, 1]
@@ -198,11 +202,17 @@ class TestCluster(unittest.TestCase):
         c2 = [1,2]
         c3 = [3]
 
-        distance = clusterdistance(data, mask=mask, weight=weight, index1=c1, index2=c2, dist='e', method='a', transpose=0);
+        distance = clusterdistance(data, mask=mask, weight=weight,
+                                   index1=c1, index2=c2, dist='e',
+                                   method='a', transpose=0);
         self.assertAlmostEqual(distance, 6.650, 3)
-        distance = clusterdistance(data, mask=mask, weight=weight, index1=c1, index2=c3, dist='e', method='a', transpose=0);
+        distance = clusterdistance(data, mask=mask, weight=weight,
+                                   index1=c1, index2=c3, dist='e',
+                                   method='a', transpose=0);
         self.assertAlmostEqual(distance, 32.508, 3)
-        distance = clusterdistance(data, mask=mask, weight=weight, index1=c2, index2=c3, dist='e', method='a', transpose=0);
+        distance = clusterdistance(data, mask=mask, weight=weight,
+                                   index1=c2, index2=c3, dist='e',
+                                   method='a', transpose=0);
         self.assertAlmostEqual(distance, 15.118, 3)
 
         # Second data set
@@ -239,11 +249,17 @@ class TestCluster(unittest.TestCase):
         c2 = [ 4, 5, 6, 7 ]
         c3 = [ 8 ]
 
-        distance = clusterdistance(data, mask=mask, weight=weight, index1=c1, index2=c2, dist='e', method='a', transpose=0);
+        distance = clusterdistance(data, mask=mask, weight=weight,
+                                   index1=c1, index2=c2, dist='e',
+                                   method='a', transpose=0);
         self.assertAlmostEqual(distance, 5.833, 3)
-        distance = clusterdistance(data, mask=mask, weight=weight, index1=c1, index2=c3, dist='e', method='a', transpose=0);
+        distance = clusterdistance(data, mask=mask, weight=weight,
+                                   index1=c1, index2=c3, dist='e',
+                                   method='a', transpose=0);
         self.assertAlmostEqual(distance, 3.298, 3)
-        distance = clusterdistance(data, mask=mask, weight=weight, index1=c2, index2=c3, dist='e', method='a', transpose=0);
+        distance = clusterdistance(data, mask=mask, weight=weight,
+                                   index1=c2, index2=c3, dist='e',
+                                   method='a', transpose=0);
         self.assertAlmostEqual(distance, 0.360, 3)
 
 
@@ -266,7 +282,8 @@ class TestCluster(unittest.TestCase):
       
         # test first data set
         # Pairwise average-linkage clustering"
-        tree = treecluster(data=data1, mask=mask1, weight=weight1, transpose=0, method='a', dist='e')
+        tree = treecluster(data=data1, mask=mask1, weight=weight1,
+                           transpose=0, method='a', dist='e')
         self.assertEqual(len(tree), len(data1)-1)
         self.assertEqual(tree[0].left, 2)
         self.assertEqual(tree[0].right, 1)
@@ -279,7 +296,8 @@ class TestCluster(unittest.TestCase):
         self.assertAlmostEqual(tree[2].distance, 21.348, 3)
 
         # Pairwise single-linkage clustering
-        tree = treecluster(data=data1, mask=mask1, weight=weight1, transpose=0, method='s', dist='e')
+        tree = treecluster(data=data1, mask=mask1, weight=weight1,
+                           transpose=0, method='s', dist='e')
         self.assertEqual(len(tree), len(data1)-1)
         self.assertEqual(tree[0].left, 1)
         self.assertEqual(tree[0].right, 2)
@@ -292,7 +310,8 @@ class TestCluster(unittest.TestCase):
         self.assertAlmostEqual(tree[2].distance, 12.908, 3)
 
         # Pairwise centroid-linkage clustering
-        tree = treecluster(data=data1, mask=mask1, weight=weight1, transpose=0, method='c', dist='e')
+        tree = treecluster(data=data1, mask=mask1, weight=weight1,
+                           transpose=0, method='c', dist='e')
         self.assertEqual(len(tree), len(data1)-1)
         self.assertEqual(tree[0].left, 1)
         self.assertEqual(tree[0].right, 2)
@@ -305,7 +324,8 @@ class TestCluster(unittest.TestCase):
         self.assertAlmostEqual(tree[2].distance, 19.437, 3)
 
         # Pairwise maximum-linkage clustering
-        tree = treecluster(data=data1, mask=mask1, weight=weight1, transpose=0, method='m', dist='e')
+        tree = treecluster(data=data1, mask=mask1, weight=weight1,
+                           transpose=0, method='m', dist='e')
         self.assertEqual(len(tree), len(data1)-1)
         self.assertEqual(tree[0].left, 2)
         self.assertEqual(tree[0].right, 1)
@@ -348,7 +368,8 @@ class TestCluster(unittest.TestCase):
       
         # Test second data set
         # Pairwise average-linkage clustering
-        tree = treecluster(data=data2, mask=mask2, weight=weight2, transpose=0, method='a', dist='e')
+        tree = treecluster(data=data2, mask=mask2, weight=weight2,
+                           transpose=0, method='a', dist='e')
         self.assertEqual(len(tree), len(data2)-1)
         self.assertEqual(tree[0].left, 5)
         self.assertEqual(tree[0].right, 4)
@@ -388,7 +409,8 @@ class TestCluster(unittest.TestCase):
         self.assertAlmostEqual(tree[11].distance, 12.741, 3)
       
         # Pairwise single-linkage clustering
-        tree = treecluster(data=data2, mask=mask2, weight=weight2, transpose=0, method='s', dist='e')
+        tree = treecluster(data=data2, mask=mask2, weight=weight2,
+                           transpose=0, method='s', dist='e')
         self.assertEqual(len(tree), len(data2)-1)
         self.assertEqual(tree[0].left, 4)
         self.assertEqual(tree[0].right, 5)
@@ -428,7 +450,8 @@ class TestCluster(unittest.TestCase):
         self.assertAlmostEqual(tree[11].distance, 3.535, 3)
       
         # Pairwise centroid-linkage clustering
-        tree = treecluster(data=data2, mask=mask2, weight=weight2, transpose=0, method='c', dist='e')
+        tree = treecluster(data=data2, mask=mask2, weight=weight2,
+                           transpose=0, method='c', dist='e')
         self.assertEqual(len(tree), len(data2)-1)
         self.assertEqual(tree[0].left, 4)
         self.assertEqual(tree[0].right, 5)
@@ -468,7 +491,8 @@ class TestCluster(unittest.TestCase):
         self.assertAlmostEqual(tree[11].distance, 11.536, 3)
       
         # Pairwise maximum-linkage clustering
-        tree = treecluster(data=data2, mask=mask2, weight=weight2, transpose=0, method='m', dist='e')
+        tree = treecluster(data=data2, mask=mask2, weight=weight2,
+                           transpose=0, method='m', dist='e')
         self.assertEqual(len(tree), len(data2)-1)
         self.assertEqual(tree[0].left, 5)
         self.assertEqual(tree[0].right, 4)
@@ -524,7 +548,9 @@ class TestCluster(unittest.TestCase):
                             [ 1, 1, 1, 1, 1], 
                             [ 1, 1, 1, 1, 1]], int)
 
-        clusterid, celldata = somcluster(data=data, mask=mask, weight=weight, transpose=0, nxgrid=10, nygrid=10, inittau=0.02, niter=100, dist='e')
+        clusterid, celldata = somcluster(data=data, mask=mask, weight=weight,
+                                         transpose=0, nxgrid=10, nygrid=10,
+                                         inittau=0.02, niter=100, dist='e')
         self.assertEqual(len(clusterid), len(data))
         self.assertEqual(len(clusterid[0]), 2)
 
@@ -557,7 +583,9 @@ class TestCluster(unittest.TestCase):
                             [ 1, 1 ],
                             [ 1, 1 ]], int)
 
-        clusterid, celldata = somcluster(data=data, mask=mask, weight=weight, transpose=0, nxgrid=10, nygrid=10, inittau=0.02, niter=100, dist='e')
+        clusterid, celldata = somcluster(data=data, mask=mask, weight=weight,
+                                         transpose=0, nxgrid=10, nygrid=10,
+                                         inittau=0.02, niter=100, dist='e')
         self.assertEqual(len(clusterid), len(data))
         self.assertEqual(len(clusterid[0]), 2)
 
@@ -643,6 +671,7 @@ class TestCluster(unittest.TestCase):
         self.assertEqual(clusterid[8], 2)
         self.assertAlmostEqual(error, 7.680, 3)
 
-if __name__ == "__main__" :
-    print "test_Cluster"
-    run_tests("Bio.Cluster")
+if __name__ == "__main__":
+    TestCluster.module = 'Bio.Cluster'
+    runner = unittest.TextTestRunner(verbosity = 2)
+    unittest.main(testRunner=runner)
