@@ -1,10 +1,15 @@
-#!/usr/bin/env python
+# This code is part of the Biopython distribution and governed by its
+# license.  Please see the LICENSE file that should have been included
+# as part of this package.
 
-import StringIO
-from operator import truth
+try:
+    from numpy import asarray
+except ImportError:
+    from Bio import MissingExternalDependencyError
+    raise MissingExternalDependencyError(\
+        "Install NumPy if you want to use Bio.MarkovModel.")
 
 from Bio import MarkovModel
-from Numeric import ones, zeros, log, asarray
 
 def print_mm(markov_model):
     print "STATES: %s" % ' '.join(markov_model.states)
@@ -36,7 +41,14 @@ training_data = [
 print "Training HMM"
 mm = MarkovModel.train_visible(states, alphabet, training_data)
 print "Classifying"
-print MarkovModel.find_states(mm, "AACGTT")
+
+#print MarkovModel.find_states(mm, "AACGTT")
+#Don't just print this, as the float may have different
+#precision on different platforms.  This returns a list
+#containing a tuple containing a list (fine), and a float.
+states = MarkovModel.find_states(mm, "AACGTT")
+for state_list, state_float in states:
+    print "State %s, %0.10f" % (repr(state_list), state_float)
 print_mm(mm)
 
 
