@@ -14,19 +14,11 @@ import unittest
 from Bio.SCOP import Dom
 from Bio.SCOP.Residues import Residues
 
-import sys
 
-def run_tests(argv):
-    test_suite = testing_suite()
-    runner = unittest.TextTestRunner(sys.stdout, verbosity = 2)
-    runner.run(test_suite)
-
-def testing_suite():
-    return test_suite()
 
 
 class DomTests(unittest.TestCase):
-    def setUp(self) :
+    def setUp(self):
         self.filename = './SCOP/testDom.txt'
 
     def testParse(self):
@@ -49,12 +41,12 @@ class DomTests(unittest.TestCase):
        finally:
            f.close()
 
-    def testError(self) :
+    def testError(self):
         corruptDom = "49xxx268\tsp\tb.1.2.1\t-\n"
         self.assertRaises(ValueError, Dom.Record, corruptDom)
 
 
-    def testRecord(self) :
+    def testRecord(self):
         recLine = 'd7hbib_\t7hbi\tb:\t1.001.001.001.001.001'
 
         rec = Dom.Record(recLine)
@@ -64,15 +56,7 @@ class DomTests(unittest.TestCase):
         self.assertEquals(rec.hierarchy,'1.001.001.001.001.001')
 
 
-def test_suite():
-    return unittest.makeSuite(DomTests)
 
 if __name__ == '__main__':
-    unittest.main()
-
-
-
-
-
-
-
+    runner = unittest.TextTestRunner(verbosity = 2)
+    unittest.main(testRunner=runner)
