@@ -11,20 +11,12 @@ import unittest
 from Bio.SCOP import Hie
 from Bio.SCOP.Residues import Residues
 
-import sys
 
-def run_tests(argv):
-    test_suite = testing_suite()
-    runner = unittest.TextTestRunner(sys.stdout, verbosity = 2)
-    runner.run(test_suite)
-
-def testing_suite():
-    return test_suite()
 
 
 class HieTests(unittest.TestCase):
 
-    def setUp(self) :
+    def setUp(self):
         self.filename = './SCOP/dir.hie.scop.txt_test'
 
     def testParse(self):
@@ -47,17 +39,16 @@ class HieTests(unittest.TestCase):
        finally:
            f.close()        
 
-    def testError(self) :
+    def testError(self):
         corruptRec = "4926sdfhjhfgyjdfyg"
 
         try:
             rec = Hie.Record(corruptRec)
             assert False, "Should never get here"
-        except ValueError, e :
+        except ValueError, e:
             pass
 
-def test_suite():
-    return unittest.makeSuite(HieTests)
 
 if __name__ == '__main__':
-    unittest.main()
+    runner = unittest.TextTestRunner(verbosity = 2)
+    unittest.main(testRunner=runner)
