@@ -10,27 +10,6 @@ from StringIO import *
 
 from Bio.SCOP import *
 
-import sys
-
-def run_tests(argv):
-    test_suite = testing_suite()
-    runner = unittest.TextTestRunner(sys.stdout, verbosity = 2)
-    runner.run(test_suite)
-
-def testing_suite():
-    """Generate the suite of tests.
-    """
-    test_suite = unittest.TestSuite()
-    
-    test_loader = unittest.TestLoader()
-    test_loader.testMethodPrefix = 'test'
-    tests = [ScopTests]
-    
-    for test in tests:
-        cur_suite = test_loader.loadTestsFromTestCase(test)
-        test_suite.addTest(cur_suite)
-        
-    return test_suite
 
 
 class ScopTests(unittest.TestCase):
@@ -81,7 +60,7 @@ class ScopTests(unittest.TestCase):
                 
 
 
-    def testSccsOrder(self) :
+    def testSccsOrder(self):
         assert cmp_sccs("a.1.1.1", "a.1.1.1") == 0
         assert cmp_sccs("a.1.1.2", "a.1.1.1") == 1
         assert cmp_sccs("a.1.1.2", "a.1.1.11") == -1
@@ -90,7 +69,7 @@ class ScopTests(unittest.TestCase):
         assert cmp_sccs("b.1.2.2", "a.5.1.11") == 1
         assert cmp_sccs("b.1.2.2", "b.1.2") == 1        
 
-    def testParseDomain(self) :
+    def testParseDomain(self):
         s=">d1tpt_1 a.46.2.1 (1-70) Thymidine phosphorylase {Escherichia coli}"
         dom = parse_domain(s)
 
@@ -118,7 +97,7 @@ class ScopTests(unittest.TestCase):
         try:
             dom = parse_domain("Totally wrong")
             assert False, "Should never get here"
-        except ValueError, e :
+        except ValueError, e:
             pass
 
     def testConstructFromDirectory(self):
@@ -171,13 +150,7 @@ class ScopTests(unittest.TestCase):
         
         
 
-#def test_suite():
-#    return unittest.makeSuite(ScopTests)
 
 if __name__ == '__main__':
-    unittest.main()
-
-
-
-
-
+    runner = unittest.TextTestRunner(verbosity = 2)
+    unittest.main(testRunner=runner)
