@@ -1,8 +1,6 @@
-#TODO - Remove this work around once we drop python 2.3 support
-try:
-    set = set
-except NameError:
-    from sets import Set as set
+# This code is part of the Biopython distribution and governed by its
+# license.  Please see the LICENSE file that should have been included
+# as part of this package.
 
 from Bio import Seq
 from Bio.Alphabet import IUPAC
@@ -133,113 +131,116 @@ print
 print "Testing Seq addition"
 print "===================="
 dna = [Seq.Seq("ATCG", IUPAC.ambiguous_dna),
-       Seq.Seq("GTCA", Alphabet.generic_dna),
+       Seq.Seq("gtca", Alphabet.generic_dna),
        Seq.MutableSeq("GGTCA", Alphabet.generic_dna),
        Seq.Seq("CTG-CA", Alphabet.Gapped(IUPAC.unambiguous_dna, "-")),
        "TGGTCA"]
 rna = [Seq.Seq("AUUUCG", IUPAC.ambiguous_rna),
        Seq.MutableSeq("AUUCG", IUPAC.ambiguous_rna),
-       Seq.Seq("UCAG", Alphabet.generic_rna),
+       Seq.Seq("uCAg", Alphabet.generic_rna),
        Seq.MutableSeq("UC-AG", Alphabet.Gapped(Alphabet.generic_rna, "-")),
        Seq.Seq("U.CAG", Alphabet.Gapped(Alphabet.generic_rna, ".")),
        "UGCAU"]
 nuc = [Seq.Seq("ATCG", Alphabet.generic_nucleotide),"UUUTTTACG"]
 protein = [Seq.Seq("ATCGPK", IUPAC.protein),
+           Seq.Seq("atcGPK", Alphabet.generic_protein),
            Seq.Seq("T.CGPK", Alphabet.Gapped(IUPAC.protein, ".")),
            Seq.Seq("T-CGPK", Alphabet.Gapped(IUPAC.protein, "-")),
            Seq.Seq("MEDG-KRXR*", Alphabet.Gapped(Alphabet.HasStopCodon(IUPAC.extended_protein, "*"), "-")),
            Seq.MutableSeq("ME-K-DRXR*XU", Alphabet.Gapped(Alphabet.HasStopCodon(IUPAC.extended_protein, "*"), "-")),
-           Seq.Seq("MEDG-KRXR@", Alphabet.HasStopCodon(Alphabet.Gapped(IUPAC.extended_protein, "-"), "*")),
+           Seq.Seq("MEDG-KRXR@", Alphabet.HasStopCodon(Alphabet.Gapped(IUPAC.extended_protein, "-"), "@")),
            Seq.Seq("ME-KR@", Alphabet.HasStopCodon(Alphabet.Gapped(IUPAC.protein, "-"), "@")),
            Seq.Seq("MEDG.KRXR@", Alphabet.Gapped(Alphabet.HasStopCodon(IUPAC.extended_protein, "@"), ".")),
            "TEDDF"]
-for a in dna+rna :
-    for b in nuc :
+for a in dna+rna:
+    for b in nuc:
         c=a+b
         assert str(c) == str(a) + str(b)
-for a in rna :
-    for b in rna :
-        try :
-            c=a+b
-            assert str(c) == str(a) + str(b)
-        except ValueError, e :
-            print "%s + %s\n-> %s" % (repr(a.alphabet), repr(b.alphabet), str(e))
-for a in dna :
-    for b in dna :
-        try :
-            c=a+b
-            assert str(c) == str(a) + str(b)
-        except ValueError, e :
-            print "%s + %s\n-> %s" % (repr(a.alphabet), repr(b.alphabet), str(e))
-    for b in rna :
-        try :
-            c=a+b
-            assert (isinstance(a,str) or isinstance(b,str)), \
-                   "DNA+RNA addition should fail!"
-        except TypeError :
-            pass
-        try :
-            c=b+a
-            assert (isinstance(a,str) or isinstance(b,str)), \
-                   "RNA+DNA addition should fail!"
-        except TypeError :
-            pass
-for a in protein :
-    for b in protein :
-        try :
+for a in rna:
+    for b in rna:
+        try:
             c=a+b
             assert str(c) == str(a) + str(b)
         except ValueError, e:
             print "%s + %s\n-> %s" % (repr(a.alphabet), repr(b.alphabet), str(e))
-    for b in nuc+dna+rna :
-        try :
+for a in dna:
+    for b in dna:
+        try:
+            c=a+b
+            assert str(c) == str(a) + str(b)
+        except ValueError, e:
+            print "%s + %s\n-> %s" % (repr(a.alphabet), repr(b.alphabet), str(e))
+    for b in rna:
+        try:
+            c=a+b
+            assert (isinstance(a,str) or isinstance(b,str)), \
+                   "DNA+RNA addition should fail!"
+        except TypeError:
+            pass
+        try:
+            c=b+a
+            assert (isinstance(a,str) or isinstance(b,str)), \
+                   "RNA+DNA addition should fail!"
+        except TypeError:
+            pass
+for a in protein:
+    for b in protein:
+        try:
+            c=a+b
+            assert str(c) == str(a) + str(b)
+        except ValueError, e:
+            print "%s + %s\n-> %s" % (repr(a.alphabet), repr(b.alphabet), str(e))
+    for b in nuc+dna+rna:
+        try:
             c=a+b
             assert (isinstance(a,str) or isinstance(b,str)), \
                    "Protein+Nucleotide addition should fail!"
-        except TypeError :
+        except TypeError:
             pass
-for a in nuc :
-    for b in dna+rna+nuc :
+for a in nuc:
+    for b in dna+rna+nuc:
         c=a+b
         assert str(c) == str(a) + str(b)
-for a in dna+rna+nuc :
-    for b in protein :
-        try :
+for a in dna+rna+nuc:
+    for b in protein:
+        try:
             c=a+b
             assert (isinstance(a,str) or isinstance(b,str)), \
                    "Nucleotide+Protein addition should fail!"
-        except TypeError :
+        except TypeError:
             pass
 
 ###########################################################################
 print
 print "Testing Seq string methods"
 print "=========================="
-for a in dna + rna + nuc + protein :
+for a in dna + rna + nuc + protein:
     if not isinstance(a, Seq.Seq) : continue
     assert a.strip().tostring() == a.tostring().strip()
     assert a.lstrip().tostring() == a.tostring().lstrip()
     assert a.rstrip().tostring() == a.tostring().rstrip()
+    assert a.lower().tostring() == a.tostring().lower()
+    assert a.upper().tostring() == a.tostring().upper()
     test_chars = ["-", Seq.Seq("-"), Seq.Seq("*"), "-X@"]
     alpha = Alphabet._get_base_alphabet(a.alphabet)
-    if isinstance(alpha, Alphabet.DNAAlphabet) :
+    if isinstance(alpha, Alphabet.DNAAlphabet):
         test_chars.append(Seq.Seq("A", IUPAC.ambiguous_dna))
-    if isinstance(alpha, Alphabet.RNAAlphabet) :
+    if isinstance(alpha, Alphabet.RNAAlphabet):
         test_chars.append(Seq.Seq("A", IUPAC.ambiguous_rna))
-    if isinstance(alpha, Alphabet.NucleotideAlphabet) :
+    if isinstance(alpha, Alphabet.NucleotideAlphabet):
         test_chars.append(Seq.Seq("A", Alphabet.generic_nucleotide))
-    if isinstance(alpha, Alphabet.ProteinAlphabet) :
+    if isinstance(alpha, Alphabet.ProteinAlphabet):
         test_chars.append(Seq.Seq("K", Alphabet.generic_protein))
         test_chars.append(Seq.Seq("K-", Alphabet.Gapped(Alphabet.generic_protein,"-")))
         test_chars.append(Seq.Seq("K@", Alphabet.Gapped(IUPAC.protein,"@")))
         #Setup a clashing alphabet sequence
         b = Seq.Seq("-", Alphabet.generic_nucleotide)
-    else :
+    else:
         b = Seq.Seq("-", Alphabet.generic_protein)
-    try :
+    try:
         print a.strip(b).tostring()
         assert False, "Alphabet should have clashed!"
-    except TypeError :
+    except TypeError:
         pass #Good!
             
     for chars in  test_chars:
@@ -256,21 +257,13 @@ for a in dna + rna + nuc + protein :
         #Now check splits
         assert [x.tostring() for x in a.split(chars)] \
                == a.tostring().split(str(chars))
-        try :
-            assert [x.tostring() for x in a.rsplit(chars)] \
-                   == a.tostring().rsplit(str(chars))
-        except AttributeError :
-            #Python 2.3 string lacks the rsplit method
-            pass
-        for max_sep in [0,1,2,999] :
+        assert [x.tostring() for x in a.rsplit(chars)] \
+               == a.tostring().rsplit(str(chars))
+        for max_sep in [0,1,2,999]:
             assert [x.tostring() for x in a.split(chars, max_sep)] \
                    == a.tostring().split(str(chars), max_sep)
-            try :
-                assert [x.tostring() for x in a.rsplit(chars, max_sep)] \
-                       == a.tostring().rsplit(str(chars), max_sep)
-            except AttributeError :
-                #Python 2.3 string lacks the rsplit method
-                pass
+            assert [x.tostring() for x in a.rsplit(chars, max_sep)] \
+                   == a.tostring().rsplit(str(chars), max_sep)
 del a, alpha, chars, str_chars, test_chars
 del dna, rna, nuc, protein
 ###########################################################################
@@ -282,24 +275,29 @@ print "=============================="
 assert "-" not in ambiguous_dna_values
 assert "?" not in ambiguous_dna_values
 
-def complement(sequence) :
+def complement(sequence):
     #TODO - Add a complement function to Bio/Seq.py?
     #There is already a complement method on the Seq and MutableSeq objects.
     return Seq.reverse_complement(sequence)[::-1]
 
+def sorted_dict(d):
+    """A sorted repr of a dictionary."""
+    return "{%s}" % ", ".join("%s: %s" % (repr(k),repr(v)) \
+                              for k,v in sorted(d.iteritems()))
+
 print
-print "DNA Ambiguity mapping:", ambiguous_dna_values
-print "DNA Complement mapping:", ambiguous_dna_complement
-for ambig_char, values in ambiguous_dna_values.iteritems() :
+print "DNA Ambiguity mapping:", sorted_dict(ambiguous_dna_values)
+print "DNA Complement mapping:", sorted_dict(ambiguous_dna_complement)
+for ambig_char, values in sorted(ambiguous_dna_values.iteritems()):
     compl_values = complement(values)
     print "%s={%s} --> {%s}=%s" % \
         (ambig_char, values, compl_values, ambiguous_dna_complement[ambig_char])
     assert set(compl_values) == set(ambiguous_dna_values[ambiguous_dna_complement[ambig_char]])
     
 print
-print "RNA Ambiguity mapping:", ambiguous_rna_values
-print "RNA Complement mapping:", ambiguous_rna_complement
-for ambig_char, values in ambiguous_rna_values.iteritems() :
+print "RNA Ambiguity mapping:", sorted_dict(ambiguous_rna_values)
+print "RNA Complement mapping:", sorted_dict(ambiguous_rna_complement)
+for ambig_char, values in sorted(ambiguous_rna_values.iteritems()):
     compl_values = complement(values).replace("T","U") #need to help as no alphabet
     print "%s={%s} --> {%s}=%s" % \
         (ambig_char, values, compl_values, ambiguous_rna_complement[ambig_char])
@@ -307,12 +305,12 @@ for ambig_char, values in ambiguous_rna_values.iteritems() :
 
 print
 print "Reverse complements:"
-for sequence in [Seq.Seq("".join(ambiguous_rna_values)),
-            Seq.Seq("".join(ambiguous_dna_values)),
-            Seq.Seq("".join(ambiguous_rna_values), Alphabet.generic_rna),
-            Seq.Seq("".join(ambiguous_dna_values), Alphabet.generic_dna),
-            Seq.Seq("".join(ambiguous_rna_values), IUPAC.IUPACAmbiguousRNA()),
-            Seq.Seq("".join(ambiguous_dna_values), IUPAC.IUPACAmbiguousDNA()),
+for sequence in [Seq.Seq("".join(sorted(ambiguous_rna_values))),
+            Seq.Seq("".join(sorted(ambiguous_dna_values))),
+            Seq.Seq("".join(sorted(ambiguous_rna_values)), Alphabet.generic_rna),
+            Seq.Seq("".join(sorted(ambiguous_dna_values)), Alphabet.generic_dna),
+            Seq.Seq("".join(sorted(ambiguous_rna_values)).replace("X",""), IUPAC.IUPACAmbiguousRNA()),
+            Seq.Seq("".join(sorted(ambiguous_dna_values)).replace("X",""), IUPAC.IUPACAmbiguousDNA()),
             Seq.Seq("AWGAARCKG")]:  # Note no U or T
         print "%s -> %s" \
               % (repr(sequence), repr(Seq.reverse_complement(sequence)))
@@ -354,16 +352,16 @@ protein_seqs = [Seq.Seq("ATCGPK", IUPAC.protein),
                 Seq.Seq("T-CGPK", Alphabet.Gapped(IUPAC.protein, "-")),
                 Seq.Seq("MEDG-KRXR*", Alphabet.Gapped(Alphabet.HasStopCodon(IUPAC.extended_protein, "*"), "-")),
                 Seq.MutableSeq("ME-K-DRXR*XU", Alphabet.Gapped(Alphabet.HasStopCodon(IUPAC.extended_protein, "*"), "-")),
-                Seq.Seq("MEDG-KRXR@", Alphabet.HasStopCodon(Alphabet.Gapped(IUPAC.extended_protein, "-"), "*")),
+                Seq.Seq("MEDG-KRXR@", Alphabet.HasStopCodon(Alphabet.Gapped(IUPAC.extended_protein, "-"), "@")),
                 Seq.Seq("ME-KR@", Alphabet.HasStopCodon(Alphabet.Gapped(IUPAC.protein, "-"), "@")),
                 Seq.Seq("MEDG.KRXR@", Alphabet.Gapped(Alphabet.HasStopCodon(IUPAC.extended_protein, "@"), "."))]
 
 #Sanity test on the test sequence alphabets (see also enhancement bug 2597)
-for nucleotide_seq in test_seqs :
-    if hasattr(nucleotide_seq, "alphabet") :
-        if "U" in str(nucleotide_seq).upper() :
+for nucleotide_seq in test_seqs:
+    if hasattr(nucleotide_seq, "alphabet"):
+        if "U" in str(nucleotide_seq).upper():
             assert not isinstance(nucleotide_seq.alphabet, Alphabet.DNAAlphabet)
-        if "T" in str(nucleotide_seq).upper() :
+        if "T" in str(nucleotide_seq).upper():
             assert not isinstance(nucleotide_seq.alphabet, Alphabet.RNAAlphabet)
             
 
@@ -371,150 +369,150 @@ print
 print "Transcribe DNA into RNA"
 print "======================="
 for nucleotide_seq in test_seqs:
-    try :
+    try:
         expected = Seq.transcribe(nucleotide_seq)
         assert str(nucleotide_seq).replace("t","u").replace("T","U") == str(expected)
         print "%s -> %s" \
         % (repr(nucleotide_seq) , repr(expected))
-    except ValueError, e :
+    except ValueError, e:
         expected = None
         print "%s -> %s" \
         % (repr(nucleotide_seq) , str(e))
     #Now test the Seq object's method
-    if isinstance(nucleotide_seq, Seq.Seq) :
-        try :
+    if isinstance(nucleotide_seq, Seq.Seq):
+        try:
             assert repr(expected) == repr(nucleotide_seq.transcribe())
-        except ValueError :
+        except ValueError:
             assert expected is None
 
-for s in protein_seqs :
-    try :
+for s in protein_seqs:
+    try:
         print Seq.transcribe(s)
         assert False, "Transcription shouldn't work on a protein!"
-    except ValueError :
+    except ValueError:
         pass
     if not isinstance(s, Seq.Seq) : continue #Only Seq has this method
-    try :
+    try:
         print s.transcribe()
         assert False, "Transcription shouldn't work on a protein!"
-    except ValueError :
+    except ValueError:
         pass
 
 print
 print "Back-transcribe RNA into DNA"
 print "============================"
 for nucleotide_seq in test_seqs:
-    try :
+    try:
         expected = Seq.back_transcribe(nucleotide_seq)
         assert str(nucleotide_seq).replace("u","t").replace("U","T") == str(expected)
         print "%s -> %s" \
         % (repr(nucleotide_seq) , repr(expected))
-    except ValueError, e :
+    except ValueError, e:
         expected = None
         print "%s -> %s" \
         % (repr(nucleotide_seq) , str(e))
     #Now test the Seq object's method
-    if isinstance(nucleotide_seq, Seq.Seq) :
-        try :
+    if isinstance(nucleotide_seq, Seq.Seq):
+        try:
             assert repr(expected) == repr(nucleotide_seq.back_transcribe())
-        except ValueError :
+        except ValueError:
             assert expected is None
             
-for s in protein_seqs :
-    try :
+for s in protein_seqs:
+    try:
         print Seq.back_transcribe(s)
         assert False, "Back transcription shouldn't work on a protein!"
-    except ValueError :
+    except ValueError:
         pass
     if not isinstance(s, Seq.Seq) : continue #Only Seq has this method
-    try :
+    try:
         print s.back_transcribe()
         assert False, "Back transcription shouldn't work on a protein!"
-    except ValueError :
+    except ValueError:
         pass
         
 print
 print "Reverse Complement"
 print "=================="
 for nucleotide_seq in test_seqs:
-    try :
+    try:
         expected = Seq.reverse_complement(nucleotide_seq)
         print "%s\n-> %s" \
         % (repr(nucleotide_seq) , repr(expected))
-    except ValueError, e :
+    except ValueError, e:
         expected = None
         print "%s\n-> %s" \
         % (repr(nucleotide_seq) , str(e))
     #Now test the Seq object's method
     #(The MutualSeq object acts in place)
-    if isinstance(nucleotide_seq, Seq.Seq) :
-        try :
+    if isinstance(nucleotide_seq, Seq.Seq):
+        try:
             assert repr(expected) == repr(nucleotide_seq.reverse_complement())
             assert repr(expected[::-1]) == repr(nucleotide_seq.complement())
-        except ValueError :
+        except ValueError:
             assert expected is None
 
-for s in protein_seqs :
-    try :
+for s in protein_seqs:
+    try:
         print Seq.reverse_complement(s)
         assert False, "Reverse complement shouldn't work on a protein!"
-    except ValueError :
+    except ValueError:
         pass
     #Note that these methods are "in place" for the MutableSeq:
-    try :
+    try:
         print s.complement()
         assert False, "Complement shouldn't work on a protein!"
-    except ValueError :
+    except ValueError:
         pass
-    try :
+    try:
         print s.reverse_complement()
         assert False, "Reverse complement shouldn't work on a protein!"
-    except ValueError :
+    except ValueError:
         pass
    
 print
 print "Translating"
 print "==========="
 for nucleotide_seq in test_seqs:
-    try :
+    try:
         expected = Seq.translate(nucleotide_seq)
         print "%s\n-> %s" \
         % (repr(nucleotide_seq) , repr(expected))
-    except (ValueError, TranslationError), e :
+    except (ValueError, TranslationError), e:
         expected = None
         print "%s\n-> %s" \
         % (repr(nucleotide_seq) , str(e))
     #Now test the Seq object's method
-    if isinstance(nucleotide_seq, Seq.Seq) :
-        try :
+    if isinstance(nucleotide_seq, Seq.Seq):
+        try:
             assert repr(expected) == repr(nucleotide_seq.translate())
-        except (ValueError, TranslationError) :
+        except (ValueError, TranslationError):
             assert expected is None
     #Now check translate(..., to_stop=True)
-    try :
+    try:
         short = Seq.translate(nucleotide_seq, to_stop=True)
-    except (ValueError, TranslationError), e :
+    except (ValueError, TranslationError), e:
         short = None
-    if expected is not None :
+    if expected is not None:
         assert short is not None
         assert str(short) == str(expected.split("*")[0])
-    if isinstance(nucleotide_seq, Seq.Seq) :
-        try :
+    if isinstance(nucleotide_seq, Seq.Seq):
+        try:
             assert repr(short) == repr(nucleotide_seq.translate(to_stop=True))
-        except (ValueError, TranslationError) :
+        except (ValueError, TranslationError):
             assert short is None
 
-for s in protein_seqs :
-    try :
+for s in protein_seqs:
+    try:
         print Seq.translate(s)
         assert False, "Translation shouldn't work on a protein!"
-    except ValueError :
+    except ValueError:
         pass
     if not isinstance(s, Seq.Seq) : continue #Only Seq has this method
-    try :
+    try:
         print s.translate()
         assert False, "Translation shouldn't work on a protein!"
-    except ValueError :
+    except ValueError:
         pass
 
 
@@ -522,7 +520,7 @@ misc_stops = "TAATAGTGAAGAAGG"
 for nucleotide_seq in [misc_stops, Seq.Seq(misc_stops),
                        Seq.Seq(misc_stops, Alphabet.generic_nucleotide),
                        Seq.Seq(misc_stops, Alphabet.DNAAlphabet()),
-                       Seq.Seq(misc_stops, IUPAC.unambiguous_dna)] :
+                       Seq.Seq(misc_stops, IUPAC.unambiguous_dna)]:
     assert "***RR" == str(Seq.translate(nucleotide_seq))
     assert "***RR" == str(Seq.translate(nucleotide_seq, table=1))
     assert "***RR" == str(Seq.translate(nucleotide_seq, table="SGC0"))
@@ -537,11 +535,11 @@ for nucleotide_seq in [misc_stops, Seq.Seq(misc_stops),
     assert "***RR" == str(Seq.translate(nucleotide_seq, table='Bacterial'))
 del misc_stops
 
-for s in protein_seqs :
-    try :
+for s in protein_seqs:
+    try:
         print Seq.translate(s)
         assert False, "Shouldn't work on a protein!"
-    except ValueError :
+    except ValueError:
         pass
 
 assert Seq.translate("TAT")=="Y"
@@ -559,35 +557,35 @@ assert Seq.translate("tar")=="*"
 assert Seq.translate("tan")=="X"
 assert Seq.translate("nnn")=="X"
 
-for codon in ["TA?", "N-N", "AC_", "Ac_"] :
-    try :
+for codon in ["TA?", "N-N", "AC_", "Ac_"]:
+    try:
         print Seq.translate(codon)
         assert "Translating %s should have failed" % repr(codon)
-    except TranslationError :
+    except TranslationError:
         pass
 
 ambig = set(IUPAC.IUPACAmbiguousDNA.letters)
-for c1 in ambig :
-    for c2 in ambig :
-        for c3 in ambig :
+for c1 in ambig:
+    for c2 in ambig:
+        for c3 in ambig:
             values = set([Seq.translate(a+b+c, table=1) \
                           for a in ambiguous_dna_values[c1] \
                           for b in ambiguous_dna_values[c2] \
                           for c in ambiguous_dna_values[c3]])
             t = Seq.translate(c1+c2+c3)
-            if t=="*" :
+            if t=="*":
                 assert values == set("*")
-            elif t=="X" :
+            elif t=="X":
                 assert len(values) > 1, \
                     "translate('%s') = '%s' not '%s'" \
                     % (c1+c2+c3, t, ",".join(values))
-            elif t=="Z" :
+            elif t=="Z":
                 assert values == set("EQ")
-            elif t=="B" :
+            elif t=="B":
                 assert values == set("DN")
-            elif t=="J" :
+            elif t=="J":
                 assert values == set("LI")
-            else :
+            else:
                 assert values == set(t)
             #TODO - Use the Bio.Data.IUPACData module for the
             #ambiguous protein mappings?
@@ -597,14 +595,14 @@ print
 print "Seq's .complement() method"
 print "=========================="
 for nucleotide_seq in test_seqs:
-    if isinstance(nucleotide_seq, Seq.Seq) :
-        try :
+    if isinstance(nucleotide_seq, Seq.Seq):
+        try:
             print "%s -> %s" \
             % (repr(nucleotide_seq) , repr(nucleotide_seq.complement()))
             assert nucleotide_seq.complement().tostring() \
                 == Seq.reverse_complement(nucleotide_seq).tostring()[::-1], \
                 "Bio.Seq function and method disagree!"
-        except ValueError, e :
+        except ValueError, e:
             print "%s -> %s" \
             % (repr(nucleotide_seq) , str(e))
         
@@ -612,13 +610,13 @@ print
 print "Seq's .reverse_complement() method"
 print "=================================="
 for nucleotide_seq in test_seqs:
-    if isinstance(nucleotide_seq, Seq.Seq) :
-        try :
+    if isinstance(nucleotide_seq, Seq.Seq):
+        try:
             print "%s -> %s" \
             % (repr(nucleotide_seq) , repr(nucleotide_seq.reverse_complement()))
             assert nucleotide_seq.reverse_complement().tostring() \
                 == Seq.reverse_complement(nucleotide_seq).tostring(), \
                 "Bio.Seq function and method disagree!"
-        except ValueError, e :
+        except ValueError, e:
             print "%s -> %s" \
             % (repr(nucleotide_seq) , str(e))
